@@ -5,24 +5,25 @@ import os
 import random
 import re
 import string
+import sys
 import textwrap
-from typing import Any, Dict, List, Optional, Tuple, Union  # noqa: F401
+from typing import Dict, List, Optional, Tuple, Union  # noqa: F401
 
 import pycloudlib  # type: ignore
 from behave.model import Feature, Scenario
 from behave.runner import Context
 
 import features.cloud as cloud
-from features.util import build_debs, emit_spinner_on_travis, lxc_get_property
+from features.util import InstallationSource, build_debs, lxc_get_property
 
 ALL_SUPPORTED_SERIES = ["bionic", "focal", "xenial"]
 
-DAILY_PPA = "http://ppa.launchpad.net/ua-client/daily/ubuntu"
-DAILY_PPA_KEYID = "6E34E7116C0BC933"
+UA_PPA_TEMPLATE = "http://ppa.launchpad.net/ua-client/{}/ubuntu"
+DEFAULT_UA_PPA_KEYID = "6E34E7116C0BC933"
 
 USERDATA_BLOCK_AUTO_ATTACH_IMG = """\
 #cloud-config
-bootcmd:
+[bootcmd, once]:
  - cp /usr/bin/ua /usr/bin/ua.orig
  - 'echo "#!/bin/sh\ndate >> /root/ua-calls\n" > /usr/bin/ua'
  - chmod 755 /usr/bin/ua
@@ -33,7 +34,7 @@ bootcmd:
 # the lxd_agent on some releases
 USERDATA_RUNCMD_PIN_PPA = """\
 runcmd:
-  - "printf \\"Package: *\\nPin: release o=LP-PPA-ua-client-daily\\nPin-Priority: 1001\\n\\" > /etc/apt/preferences.d/uaclient"
+  - "printf \\"Package: *\\nPin: release o=LP-PPA-ua-client-{}\\nPin-Priority: 1001\\n\\" > /etc/apt/preferences.d/uaclient"
 """  # noqa: E501
 
 USERDATA_RUNCMD_ENABLE_PROPOSED = """
@@ -69,12 +70,14 @@ stderr:
 
 
 class UAClientBehaveConfig:
-    """Store config options for UA client behave test runs.
+    """Store config options for Pro client behave test runs.
 
     This captures the configuration in one place, so that we have a single
     source of truth for test configuration (rather than having environment
     variable handling throughout the test code).
 
+    :param cloud_credentials_path:
+        Optional path the pycloudlib file containing the cloud credentials
     :param contract_token:
         A valid contract token to use during attach scenarios
     :param contract_token_staging:
@@ -111,23 +114,13 @@ class UAClientBehaveConfig:
     # environment variable input to the appropriate Python types for use within
     # the test framework
     boolean_options = [
-        "build_pr",
         "image_clean",
         "destroy_instances",
-        "cache_source",
-        "enable_proposed",
         "ephemeral_instance",
         "snapshot_strategy",
     ]
     str_options = [
-        "aws_access_key_id",
-        "aws_secret_access_key",
-        "az_client_id",
-        "az_client_secret",
-        "az_tenant_id",
-        "az_subscription_id",
-        "gcp_credentials_path",
-        "gcp_project",
+        "cloud_credentials_path",
         "contract_token",
         "contract_token_staging",
         "contract_token_staging_expired",
@@ -137,18 +130,14 @@ class UAClientBehaveConfig:
         "reuse_image",
         "debs_path",
         "artifact_dir",
-        "ppa",
-        "ppa_keyid",
+        "install_from",
+        "custom_ppa",
+        "custom_ppa_keyid",
         "userdata_file",
         "check_version",
+        "sbuild_chroot",
     ]
     redact_options = [
-        "aws_access_key_id",
-        "aws_secret_access_key",
-        "az_client_id",
-        "az_client_secret",
-        "az_tenant_id",
-        "az_subscription_id",
         "contract_token",
         "contract_token_staging",
         "contract_token_staging_expired",
@@ -163,19 +152,9 @@ class UAClientBehaveConfig:
     def __init__(
         self,
         *,
-        aws_access_key_id: str = None,
-        aws_secret_access_key: str = None,
-        az_client_id: str = None,
-        az_client_secret: str = None,
-        az_tenant_id: str = None,
-        az_subscription_id: str = None,
-        gcp_credentials_path: str = None,
-        gcp_project: str = None,
-        build_pr: bool = False,
+        cloud_credentials_path: str = None,
         image_clean: bool = True,
         destroy_instances: bool = True,
-        cache_source: bool = True,
-        enable_proposed: bool = False,
         ephemeral_instance: bool = False,
         snapshot_strategy: bool = False,
         machine_type: str = "lxd.container",
@@ -187,24 +166,16 @@ class UAClientBehaveConfig:
         contract_token_staging_expired: str = None,
         debs_path: str = None,
         artifact_dir: str = None,
-        ppa: str = DAILY_PPA,
-        ppa_keyid: str = DAILY_PPA_KEYID,
+        install_from: InstallationSource = InstallationSource.DAILY,
+        custom_ppa: str = None,
+        custom_ppa_keyid: str = None,
         userdata_file: str = None,
         check_version: str = None,
+        sbuild_chroot: str = None,
         cmdline_tags: List = []
     ) -> None:
         # First, store the values we've detected
-        self.aws_access_key_id = aws_access_key_id
-        self.aws_secret_access_key = aws_secret_access_key
-        self.az_client_id = az_client_id
-        self.az_client_secret = az_client_secret
-        self.az_tenant_id = az_tenant_id
-        self.az_subscription_id = az_subscription_id
-        self.gcp_credentials_path = gcp_credentials_path
-        self.gcp_project = gcp_project
-        self.build_pr = build_pr
-        self.cache_source = cache_source
-        self.enable_proposed = enable_proposed
+        self.cloud_credentials_path = cloud_credentials_path
         self.ephemeral_instance = ephemeral_instance
         self.snapshot_strategy = snapshot_strategy
         self.contract_token = contract_token
@@ -219,10 +190,12 @@ class UAClientBehaveConfig:
         self.cmdline_tags = cmdline_tags
         self.debs_path = debs_path
         self.artifact_dir = artifact_dir
-        self.ppa = ppa
-        self.ppa_keyid = ppa_keyid
+        self.install_from = install_from
+        self.custom_ppa = custom_ppa
+        self.custom_ppa_keyid = custom_ppa_keyid
         self.userdata_file = userdata_file
         self.check_version = check_version
+        self.sbuild_chroot = sbuild_chroot
         self.filter_series = set(
             [
                 tag.split(".")[1]
@@ -236,12 +209,6 @@ class UAClientBehaveConfig:
                 print(" Reuse_image specified, it will not be deleted.")
 
         ignore_vars = ()  # type: Tuple[str, ...]
-        if "aws" not in self.machine_type:
-            ignore_vars += cloud.EC2.env_vars
-        if "azure" not in self.machine_type:
-            ignore_vars += cloud.Azure.env_vars
-        if "gcp" not in self.machine_type:
-            ignore_vars += cloud.GCP.env_vars
         if "pro" in self.machine_type:
             ignore_vars += (
                 "UACLIENT_BEHAVE_CONTRACT_TOKEN",
@@ -278,39 +245,38 @@ class UAClientBehaveConfig:
         timed_job_tag += "-" + random_suffix
 
         if "aws" in self.machine_type:
+            # For AWS, we need to specify on the pycloudlib config file that
+            # the AWS region must be us-east-2. The reason for that is because
+            # our image ids were captured using that region.
             self.cloud_manager = cloud.EC2(
-                aws_access_key_id,
-                aws_secret_access_key,
-                region=os.environ.get("AWS_DEFAULT_REGION", "us-east-2"),
                 machine_type=self.machine_type,
+                cloud_credentials_path=self.cloud_credentials_path,
                 tag=timed_job_tag,
                 timestamp_suffix=False,
             )
         elif "azure" in self.machine_type:
             self.cloud_manager = cloud.Azure(
-                az_client_id=az_client_id,
-                az_client_secret=az_client_secret,
-                az_tenant_id=az_tenant_id,
-                az_subscription_id=az_subscription_id,
                 machine_type=self.machine_type,
+                cloud_credentials_path=self.cloud_credentials_path,
                 tag=timed_job_tag,
                 timestamp_suffix=False,
             )
         elif "gcp" in self.machine_type:
             self.cloud_manager = cloud.GCP(
                 machine_type=self.machine_type,
+                cloud_credentials_path=self.cloud_credentials_path,
                 tag=timed_job_tag,
                 timestamp_suffix=False,
-                gcp_credentials_path=self.gcp_credentials_path,
-                gcp_project=gcp_project,
             )
         elif "lxd.vm" in self.machine_type:
             self.cloud_manager = cloud.LXDVirtualMachine(
-                machine_type=self.machine_type
+                machine_type=self.machine_type,
+                cloud_credentials_path=self.cloud_credentials_path,
             )
         else:
             self.cloud_manager = cloud.LXDContainer(
-                machine_type=self.machine_type
+                machine_type=self.machine_type,
+                cloud_credentials_path=self.cloud_credentials_path,
             )
 
         self.cloud_api = self.cloud_manager.api
@@ -331,7 +297,9 @@ class UAClientBehaveConfig:
     def from_environ(cls, config) -> "UAClientBehaveConfig":
         """Gather config options from os.environ and return a config object"""
         # First, gather all known options
-        kwargs: Dict[str, Union[str, bool, "List"]] = {}
+        kwargs = (
+            {}
+        )  # type: Dict[str, Union[str, bool, List, InstallationSource]]
         # Preserve cmdline_tags for reference
         if not config.tags.ands:
             kwargs["cmdline_tags"] = []
@@ -355,6 +323,10 @@ class UAClientBehaveConfig:
                 if kwargs[key] == "0":
                     bool_value = False
                 kwargs[key] = bool_value
+
+        if "install_from" in kwargs:
+            kwargs["install_from"] = InstallationSource(kwargs["install_from"])
+
         return cls(**kwargs)  # type: ignore
 
 
@@ -429,22 +401,8 @@ def _should_skip_tags(context: Context, tags: List) -> str:
                 curr_machine_type = ".".join(parts[idx + 1 :])
                 machine_types.append(curr_machine_type)
                 if curr_machine_type == machine_type:
-                    if machine_type.startswith("lxd"):
-                        return ""
-
-                    cloud_manager = context.config.cloud_manager
-                    if cloud_manager and cloud_manager.missing_env_vars():
-                        return "".join(
-                            (
-                                "Skipped: {} machine_type requires:\n".format(
-                                    machine_type
-                                ),
-                                *cloud_manager.format_missing_env_vars(
-                                    cloud_manager.missing_env_vars()
-                                ),
-                            )
-                        )
                     return ""
+
                 break
             if val is None:
                 return "Skipped: tag value was None: {}".format(tag)
@@ -511,10 +469,10 @@ FAILURE_FILES = (
     "/var/lib/cloud/instance/vendor-data.txt",
 )
 FAILURE_CMDS = {
-    "ua-version": ["ua", "version"],
+    "ua-version": ["pro", "version"],
     "cloud-init-analyze": ["cloud-init", "analyze", "show"],
     "cloud-init.status": ["cloud-init", "status", "--long"],
-    "status.json": ["ua", "status", "--all", "--format=json"],
+    "status.json": ["pro", "status", "--all", "--format=json"],
     "journal.log": ["journalctl", "-b", "0"],
     "systemd-analyze-blame": ["systemd-analyze", "blame"],
     "systemctl-status": ["systemctl", "status"],
@@ -585,11 +543,7 @@ def after_step(context, step):
 
 
 def after_all(context):
-    if context.config.ppa == "":
-        logging.info(
-            " No custom images to delete. UACLIENT_BEHAVE_PPA is unset."
-        )
-    elif context.config.image_clean:
+    if context.config.image_clean:
         for key, image in context.series_image_name.items():
             if key == context.series_reuse_image:
                 logging.info(
@@ -650,11 +604,10 @@ def build_debs_from_sbuild(context: Context, series: str) -> List[str]:
         logging.info(
             "--- Could not find any debs to reuse. Building it locally"
         )
-
-        with emit_spinner_on_travis("Building debs from local source... "):
-            deb_paths = build_debs(
-                series=series, cache_source=context.config.cache_source
-            )
+        deb_paths = build_debs(
+            series=series,
+            chroot=context.config.sbuild_chroot,
+        )
 
     if "pro" in context.config.machine_type:
         return deb_paths
@@ -663,8 +616,11 @@ def build_debs_from_sbuild(context: Context, series: str) -> List[str]:
 
 
 def create_instance_with_uat_installed(
-    context: Context, series: str, name: str
-) -> None:
+    context: Context,
+    series: str,
+    name: str,
+    custom_user_data: Optional[str] = None,
+) -> pycloudlib.instance.BaseInstance:
     """Create a given series lxd image with ubuntu-advantage-tools installed
 
     This will launch a container, install ubuntu-advantage-tools, and publish
@@ -676,6 +632,12 @@ def create_instance_with_uat_installed(
         it.
     :param series:
        A string representing the series name to create
+    :param name:
+       A string representing the instance name
+    :param custom_user_data:
+       A string representing custom userdata to be added to the instance
+
+    :return: A pycloudlib Instance
     """
 
     if series in context.reuse_container:
@@ -684,58 +646,25 @@ def create_instance_with_uat_installed(
             context.reuse_container[series],
         )
         return
-    ppa = context.config.ppa
-    if ppa == "":
-        image_name = context.config.cloud_manager.locate_image_name(series)
-        logging.info(
-            "--- Unset UACLIENT_BEHAVE_PPA. Using ubuntu-advantage-tools "
-            + "from image: {}".format(image_name)
-        )
-        context.series_image_name[series] = image_name
-        return
 
-    deb_paths = []
-    if context.config.build_pr:
-        deb_paths = build_debs_from_sbuild(context, series)
+    user_data = _get_user_data_for_instance(context.config, series)
+
+    if custom_user_data:
+        prefix = "" if user_data else "#cloud-config"
+        user_data += "{}\n{}".format(prefix, custom_user_data)
 
     logging.info(
-        "--- Launching VM to create a base image with updated ubuntu-advantage"
+        "--- Launching VM to create a base image with ubuntu-advantage"
     )
-
-    user_data = ""
-    ud_file = context.config.userdata_file
-    if ud_file and os.path.exists(ud_file):
-        with open(ud_file, "r") as stream:
-            user_data = stream.read()
-    if "pro" in context.config.machine_type:
-        if not user_data:
-            user_data = USERDATA_BLOCK_AUTO_ATTACH_IMG
-    if not deb_paths:
-        if not user_data:
-            user_data = "#cloud-config\n"
-
-        if context.config.enable_proposed:
-            user_data += USERDATA_RUNCMD_ENABLE_PROPOSED.format(series=series)
-        else:
-            ppa = context.config.ppa
-            ppa_keyid = context.config.ppa_keyid
-            if context.config.ppa.startswith("ppa:"):
-                ppa = (
-                    ppa.replace("ppa:", "http://ppa.launchpad.net/")
-                    + "/ubuntu"
-                )
-
-            if ppa == DAILY_PPA:
-                user_data += USERDATA_RUNCMD_PIN_PPA
-
-            user_data += USERDATA_APT_SOURCE_PPA.format(
-                ppa_url=ppa, ppa_keyid=ppa_keyid
-            )
     inst = context.config.cloud_manager.launch(
         instance_name=name, series=series, user_data=user_data
     )
     instance_id = context.config.cloud_manager.get_instance_id(inst)
 
+    deb_paths = None
+    if context.config.install_from is InstallationSource.LOCAL:
+        deb_paths = build_debs_from_sbuild(context, series)
+
     _install_uat_in_container(
         instance_id,
         series=series,
@@ -747,6 +676,61 @@ def create_instance_with_uat_installed(
     return inst
 
 
+def _get_user_data_for_instance(
+    config: UAClientBehaveConfig, series: str
+) -> str:
+    if config.install_from in (
+        InstallationSource.ARCHIVE,
+        InstallationSource.LOCAL,
+    ):
+        return ""
+
+    user_data = "#cloud-config\n"
+
+    if "pro" in config.machine_type:
+        user_data = USERDATA_BLOCK_AUTO_ATTACH_IMG
+
+    if config.userdata_file and os.path.exists(config.userdata_file):
+        with open(config.userdata_file, "r") as stream:
+            user_data = stream.read()
+
+    if config.install_from is InstallationSource.PROPOSED:
+        user_data += USERDATA_RUNCMD_ENABLE_PROPOSED.format(series=series)
+        return user_data
+
+    if config.install_from is InstallationSource.DAILY:
+        user_data += USERDATA_RUNCMD_PIN_PPA.format("daily")
+        ppa = UA_PPA_TEMPLATE.format("daily")
+        ppa_keyid = DEFAULT_UA_PPA_KEYID
+    elif config.install_from is InstallationSource.STAGING:
+        user_data += USERDATA_RUNCMD_PIN_PPA.format("staging")
+        ppa = UA_PPA_TEMPLATE.format("staging")
+        ppa_keyid = DEFAULT_UA_PPA_KEYID
+    elif config.install_from is InstallationSource.STABLE:
+        user_data += USERDATA_RUNCMD_PIN_PPA.format("stable")
+        ppa = UA_PPA_TEMPLATE.format("stable")
+        ppa_keyid = DEFAULT_UA_PPA_KEYID
+    elif config.install_from is InstallationSource.CUSTOM:
+        if not config.custom_ppa or not config.custom_ppa_keyid:
+            logging.error(
+                "UACLIENT_BEHAVE_INSTALL_FROM is set to 'custom', "
+                + "but missing UACLIENT_BEHAVE_CUSTOM_PPA or "
+                + "UACLIENT_BEHAVE_CUSTOM_PPA_KEYID"
+            )
+            sys.exit(1)
+        ppa = config.custom_ppa
+        if ppa.startswith("ppa:"):
+            ppa = ppa.replace("ppa:", "http://ppa.launchpad.net/") + "/ubuntu"
+        ppa_keyid = config.custom_ppa_keyid or ""
+
+    user_data += USERDATA_APT_SOURCE_PPA.format(
+        ppa_url=ppa, ppa_keyid=ppa_keyid
+    )
+
+    logging.info("-- user data:\n" + user_data)
+    return user_data
+
+
 def _install_uat_in_container(
     container_id: str,
     series: str,
@@ -763,13 +747,13 @@ def _install_uat_in_container(
     :param config: UAClientBehaveConfig
     :param deb_paths: Optional paths to local deb files we need to install
     """
-    cmds: List[Any] = [["systemctl", "is-system-running", "--wait"]]
+    cmds = ["systemctl is-system-running --wait"]  # type:  List[str]
 
     if deb_paths is None:
         deb_paths = []
 
     if deb_paths:
-        cmds.append(["sudo", "apt-get", "update", "-qqy"])
+        cmds.append("sudo apt-get update -qqy")
 
         deb_files = []
         inst = config.cloud_api.get_instance(container_id)
@@ -802,7 +786,7 @@ def _install_uat_in_container(
         if "pro" in machine_type:
             ua_pkg.append("ubuntu-advantage-pro")
 
-        cmds.append(["sudo", "apt-get", "update"])
+        cmds.append("sudo apt-get update")
         cmds.append(
             " ".join(
                 [
@@ -829,10 +813,10 @@ def _install_uat_in_container(
                 conf_path
             )
         )
-        cmds.append("sudo ua status --wait")
-        cmds.append("sudo ua detach --assume-yes")
+        cmds.append("sudo pro status --wait")
+        cmds.append("sudo pro detach --assume-yes")
 
-    cmds.append(["ua", "version"])
+    cmds.append("pro version")
     instance = config.cloud_api.get_instance(container_id)
     for cmd in cmds:  # type: ignore
         result = instance.execute(cmd)
