@@ -6,6 +6,7 @@ import shlex
 import subprocess
 import time
 
+import jsonschema  # type: ignore
 import yaml
 from behave import given, then, when
 from hamcrest import (
@@ -17,17 +18,28 @@ from hamcrest import (
 )
 
 from features.environment import (
+    UA_PPA_TEMPLATE,
+    build_debs_from_sbuild,
     capture_container_as_image,
     create_instance_with_uat_installed,
 )
-from features.util import SLOW_CMDS, emit_spinner_on_travis, nullcontext
-from uaclient.defaults import DEFAULT_CONFIG_FILE, DEFAULT_MACHINE_TOKEN_PATH
+from features.util import (
+    InstallationSource,
+    SafeLoaderWithoutDatetime,
+    cleanup_instance,
+)
+from uaclient.defaults import (
+    DEFAULT_CONFIG_FILE,
+    DEFAULT_PRIVATE_MACHINE_TOKEN_PATH,
+)
 from uaclient.util import DatetimeAwareJSONDecoder
 
 CONTAINER_PREFIX = "ubuntu-behave-test"
 IMAGE_BUILD_PREFIX = "ubuntu-behave-image-build"
 IMAGE_PREFIX = "ubuntu-behave-image"
 
+ERROR_CODE = "1"
+
 
 def add_test_name_suffix(context, series, prefix):
     pr_number = os.environ.get("UACLIENT_BEHAVE_JENKINS_CHANGE_ID")
@@ -46,7 +58,7 @@ def add_test_name_suffix(context, series, prefix):
 
 
 @given("a `{series}` machine with ubuntu-advantage-tools installed")
-def given_a_machine(context, series):
+def given_a_machine(context, series, custom_user_data=None):
     if series in context.reuse_container:
         context.instances = {}
         context.container_name = context.reuse_container[series]
@@ -61,29 +73,27 @@ def given_a_machine(context, series):
 
     instance_name = add_test_name_suffix(context, series, CONTAINER_PREFIX)
 
-    if context.config.snapshot_strategy:
+    if context.config.snapshot_strategy and not custom_user_data:
         if series not in context.series_image_name:
-            with emit_spinner_on_travis():
-                build_container_name = add_test_name_suffix(
-                    context, series, IMAGE_BUILD_PREFIX
-                )
-                image_inst = create_instance_with_uat_installed(
-                    context, series, build_container_name
-                )
+            build_container_name = add_test_name_suffix(
+                context, series, IMAGE_BUILD_PREFIX
+            )
+            image_inst = create_instance_with_uat_installed(
+                context, series, build_container_name, custom_user_data
+            )
 
-                image_name = add_test_name_suffix(
-                    context, series, IMAGE_PREFIX
-                )
-                image_inst_id = context.config.cloud_manager.get_instance_id(
-                    image_inst
-                )
-                image_id = capture_container_as_image(
-                    image_inst_id,
-                    image_name=image_name,
-                    cloud_api=context.config.cloud_api,
-                )
-                context.series_image_name[series] = image_id
-                image_inst.delete(wait=False)
+            image_name = add_test_name_suffix(context, series, IMAGE_PREFIX)
+            image_inst_id = context.config.cloud_manager.get_instance_id(
+                image_inst
+            )
+            image_id = capture_container_as_image(
+                image_inst_id,
+                image_name=image_name,
+                cloud_api=context.config.cloud_api,
+            )
+
+            context.series_image_name[series] = image_id
+            image_inst.delete(wait=False)
 
         inst = context.config.cloud_manager.launch(
             series=series,
@@ -93,38 +103,106 @@ def given_a_machine(context, series):
         )
     else:
         inst = create_instance_with_uat_installed(
-            context, series, instance_name
+            context, series, instance_name, custom_user_data
         )
 
+    context.series = series
     context.instances = {"uaclient": inst}
 
     context.container_name = context.config.cloud_manager.get_instance_id(
         context.instances["uaclient"]
     )
 
-    def cleanup_instance() -> None:
-        if not context.config.destroy_instances:
-            logging.info(
-                "--- Leaving instance running: {}".format(
-                    context.instances["uaclient"].name
-                )
-            )
-            return
-        try:
-            context.instances["uaclient"].delete(wait=False)
-        except RuntimeError as e:
-            logging.error(
-                "Failed to delete instance: {}\n{}".format(
-                    context.instances["uaclient"].name, str(e)
-                )
-            )
-
-    context.add_cleanup(cleanup_instance)
+    context.add_cleanup(cleanup_instance(context, "uaclient"))
     logging.info(
         "--- instance ip: {}".format(context.instances["uaclient"].ip)
     )
 
 
+@when("I take a snapshot of the machine")
+def when_i_take_a_snapshot(context):
+    cloud = context.config.cloud_manager
+    inst = context.instances["uaclient"]
+
+    snapshot = cloud.api.snapshot(inst)
+
+    context.instance_snapshot = snapshot
+
+    def cleanup_image() -> None:
+        try:
+            context.config.cloud_manager.api.delete_image(
+                context.instance_snapshot
+            )
+        except RuntimeError as e:
+            logging.error(
+                "Failed to delete image: {}\n{}".format(
+                    context.instance_snapshot, str(e)
+                )
+            )
+
+    context.add_cleanup(cleanup_image)
+
+
+@given(
+    "a `{series}` machine with ubuntu-advantage-tools installed adding this cloud-init user_data"  # noqa
+)
+def given_a_machine_with_user_data(context, series):
+    custom_user_data = context.text
+    given_a_machine(context, series, custom_user_data)
+
+
+@when("I have the `{series}` debs under test in `{dest}`")
+def when_i_have_the_debs_under_test(context, series, dest):
+    if context.config.install_from is InstallationSource.LOCAL:
+        deb_paths = build_debs_from_sbuild(context, series)
+
+        for deb_path in deb_paths:
+            tools_or_pro = "tools" if "tools" in deb_path else "pro"
+            dest_path = "{}/ubuntu-advantage-{}.deb".format(dest, tools_or_pro)
+            context.instances["uaclient"].push_file(deb_path, dest_path)
+    else:
+        if context.config.install_from is InstallationSource.PROPOSED:
+            ppa_opts = ""
+        else:
+            if context.config.install_from is InstallationSource.DAILY:
+                ppa = UA_PPA_TEMPLATE.format("daily")
+            elif context.config.install_from is InstallationSource.STAGING:
+                ppa = UA_PPA_TEMPLATE.format("staging")
+            elif context.config.install_from is InstallationSource.STABLE:
+                ppa = UA_PPA_TEMPLATE.format("stable")
+            elif context.config.install_from is InstallationSource.CUSTOM:
+                ppa = context.config.custom_ppa
+                if not ppa.startswith("ppa"):
+                    # assumes format "http://domain.name/user/ppa/ubuntu"
+                    match = re.match(r"https?://[\w.]+/([^/]+/[^/]+)", ppa)
+                    if not match:
+                        raise AssertionError(
+                            "ppa is in unsupported format: {}".format(ppa)
+                        )
+                    ppa = "ppa:{}".format(match.group(1))
+            ppa_opts = "--distro ppa --ppa {}".format(ppa)
+        download_cmd = "pull-lp-debs {} ubuntu-advantage-tools {}".format(
+            ppa_opts, series
+        )
+        when_i_run_command(
+            context, "apt-get install -y ubuntu-dev-tools", "with sudo"
+        )
+        when_i_run_command(context, download_cmd, "with sudo")
+        logging.info("Download command `{}`".format(download_cmd))
+        logging.info("stdout: {}".format(context.process.stdout))
+        logging.info("stderr: {}".format(context.process.stderr))
+        when_i_run_shell_command(
+            context,
+            "cp ubuntu-advantage-tools*.deb ubuntu-advantage-tools.deb",
+            "with sudo",
+        )
+        when_i_run_shell_command(
+            context,
+            "cp ubuntu-advantage-pro*.deb ubuntu-advantage-pro.deb",
+            "with sudo",
+        )
+
+
 @when(
     "I launch a `{series}` `{instance_name}` machine with ingress ports `{ports}`"  # noqa
 )
@@ -150,24 +228,22 @@ def launch_machine(context, series, instance_name, ports=None):
         **kwargs
     )
 
-    def cleanup_instance() -> None:
-        if not context.config.destroy_instances:
-            logging.info(
-                "--- Leaving instance running: {}".format(
-                    context.instances[instance_name].name
-                )
-            )
-            return
-        try:
-            context.instances[instance_name].delete(wait=False)
-        except RuntimeError as e:
-            logging.error(
-                "Failed to delete instance: {}\n{}".format(
-                    context.instances[instance_name].name, str(e)
-                )
-            )
+    context.add_cleanup(cleanup_instance(context, instance_name))
 
-    context.add_cleanup(cleanup_instance)
+
+@when("I launch a `{instance_name}` machine from the snapshot")
+def launch_machine_from_snapshot(context, instance_name):
+    now = datetime.datetime.now()
+    date_prefix = now.strftime("-%s%f")
+    name = CONTAINER_PREFIX + date_prefix + "-" + instance_name
+
+    context.instances[instance_name] = context.config.cloud_manager.launch(
+        context.series,
+        instance_name=name,
+        image_name=context.instance_snapshot,
+    )
+
+    context.add_cleanup(cleanup_instance(context, instance_name))
 
 
 @when("I add this text on `{file_name}` on `{instance_name}` above `{line}`")
@@ -191,7 +267,7 @@ def when_i_run_command_on_machine(context, command, user_spec, instance_name):
 
 @when("I verify `{file_name}` is empty on `{instance_name}` machine")
 def when_i_verify_file_is_empty_on_machine(context, file_name, instance_name):
-    command = 'sh -c "cat {} | wc -l"'
+    command = 'sh -c "cat {} | wc -l"'.format(file_name)
     when_i_run_command(
         context, command, user_spec="with sudo", instance_name=instance_name
     )
@@ -222,7 +298,7 @@ def when_i_retry_run_command(context, command, user_spec, exit_codes):
 
 @when("I run `{command}` `{user_spec}` and stdin `{stdin}`")
 def when_i_run_command_with_stdin(
-    context, command, user_spec, stdin, instance_name="uaclient"
+    context, command, user_spec, stdin, instancedebug_name="uaclient"
 ):
     when_i_run_command(
         context=context, command=command, user_spec=user_spec, stdin=stdin
@@ -230,19 +306,69 @@ def when_i_run_command_with_stdin(
 
 
 @when("I do a preflight check for `{contract_token}` {user_spec}")
-def when_i_preflight(context, contract_token, user_spec):
-    token = getattr(context.config, contract_token)
-    command = "ua status --simulate-with-token {}".format(token)
+def when_i_preflight(context, contract_token, user_spec, verify_return=True):
+    token = getattr(context.config, contract_token, "invalid_token")
+    command = "pro status --simulate-with-token {}".format(token)
     if user_spec == "with the all flag":
         command += " --all"
     if "formatted as" in user_spec:
         output_format = user_spec.split()[2]
         command += " --format {}".format(output_format)
     when_i_run_command(
-        context=context, command=command, user_spec="as non-root"
+        context=context,
+        command=command,
+        user_spec="as non-root",
+        verify_return=verify_return,
     )
 
 
+@when("I initiate the magic attach flow")
+def when_i_initiate_magic_attach(context):
+    when_i_run_command(
+        context=context,
+        command="pro api u.pro.attach.magic.initiate.v1",
+        user_spec="as non-root",
+    )
+
+    magic_attach_resp = json.loads(
+        context.process.stdout.strip(), cls=DatetimeAwareJSONDecoder
+    )
+
+    context.magic_token = magic_attach_resp["data"]["attributes"]["token"]
+
+
+@when("I revoke the magic attach token")
+def when_i_revoke_the_magic_attach_token(context):
+    when_i_run_command(
+        context=context,
+        command="pro api u.pro.attach.magic.revoke.v1 --args magic_token={}".format(  # noqa
+            context.magic_token
+        ),
+        user_spec="as non-root",
+    )
+
+
+@when("I wait for the magic attach token to be activated")
+def when_i_wait_for_magic_attach_token(context):
+    when_i_run_command(
+        context=context,
+        command="pro api u.pro.attach.magic.wait.v1 --args magic_token={}".format(  # noqa
+            context.magic_token
+        ),
+        user_spec="as non-root",
+    )
+
+
+@when(
+    "I verify that a preflight check for `{contract_token}` {user_spec} exits {exit_codes}"  # noqa
+)
+def when_i_attempt_preflight(context, contract_token, user_spec, exit_codes):
+    when_i_preflight(context, contract_token, user_spec, verify_return=False)
+
+    expected_codes = exit_codes.split(",")
+    assert str(context.process.returncode) in expected_codes
+
+
 @when("I run `{command}` {user_spec}")
 def when_i_run_command(
     context,
@@ -257,17 +383,9 @@ def when_i_run_command(
             "<ci-proxy-ip>", context.instances["proxy"].ip
         )
     prefix = get_command_prefix_for_user_spec(user_spec)
-    slow_cmd_spinner = nullcontext
-    for slow_cmd in SLOW_CMDS:
-        if slow_cmd in command:
-            slow_cmd_spinner = emit_spinner_on_travis
-            break
 
     full_cmd = prefix + shlex.split(command)
-    with slow_cmd_spinner():
-        result = context.instances[instance_name].execute(
-            full_cmd, stdin=stdin
-        )
+    result = context.instances[instance_name].execute(full_cmd, stdin=stdin)
 
     process = subprocess.CompletedProcess(
         args=full_cmd,
@@ -280,6 +398,9 @@ def when_i_run_command(
         logging.error("Error executing command: {}".format(command))
         logging.error("stdout: {}".format(result.stdout))
         logging.error("stderr: {}".format(result.stderr))
+    else:
+        logging.debug("stdout: {}".format(result.stdout))
+        logging.debug("stderr: {}".format(result.stderr))
 
     if verify_return:
         assert_that(process.returncode, equal_to(0))
@@ -287,14 +408,29 @@ def when_i_run_command(
     context.process = process
 
 
+@when("I run shell command `{command}` {user_spec}")
+def when_i_run_shell_command(context, command, user_spec):
+    when_i_run_command(context, 'sh -c "{}"'.format(command), user_spec)
+
+
 @when("I fix `{issue}` by attaching to a subscription with `{token_type}`")
 def when_i_fix_a_issue_by_attaching(context, issue, token_type):
     token = getattr(context.config, token_type)
+
+    if (
+        token_type == "contract_token_staging"
+        or token_type == "contract_token_staging_expired"
+    ):
+        change_contract_endpoint_to_staging(context, user_spec="with sudo")
+    else:
+        change_contract_endpoint_to_production(context, user_spec="with sudo")
+
     when_i_run_command(
         context=context,
-        command="ua fix {}".format(issue),
+        command="pro fix {}".format(issue),
         user_spec="with sudo",
         stdin="a\n{}\n".format(token),
+        verify_return=False,
     )
 
 
@@ -302,7 +438,7 @@ def when_i_fix_a_issue_by_attaching(context, issue, token_type):
 def when_i_fix_a_issue_by_enabling_service(context, issue):
     when_i_run_command(
         context=context,
-        command="ua fix {}".format(issue),
+        command="pro fix {}".format(issue),
         user_spec="with sudo",
         stdin="e\n",
     )
@@ -313,7 +449,7 @@ def when_i_fix_a_issue_by_updating_expired_token(context, issue):
     token = getattr(context.config, "contract_token")
     when_i_run_command(
         context=context,
-        command="ua fix {}".format(issue),
+        command="pro fix {}".format(issue),
         user_spec="with sudo",
         stdin="r\n{}\n".format(token),
     )
@@ -334,29 +470,53 @@ def when_i_update_contract_field_to_new_value(
             contract_field,
             contract_field,
             new_value,
-            DEFAULT_MACHINE_TOKEN_PATH,
+            DEFAULT_PRIVATE_MACHINE_TOKEN_PATH,
         ),
         user_spec="with sudo",
     )
 
 
+@when("I change contract to staging {user_spec}")
+def change_contract_endpoint_to_staging(context, user_spec):
+    when_i_run_command(
+        context,
+        "sed -i 's/contracts.can/contracts.staging.can/' {}".format(
+            DEFAULT_CONFIG_FILE
+        ),
+        user_spec,
+    )
+
+
+def change_contract_endpoint_to_production(context, user_spec):
+    when_i_run_command(
+        context,
+        "sed -i 's/contracts.staging.can/contracts.can/' {}".format(
+            DEFAULT_CONFIG_FILE
+        ),
+        user_spec,
+    )
+
+
+@when("I attach `{token_type}` {user_spec} and options `{options}`")
+def when_i_attach_staging_token_with_options(
+    context, token_type, user_spec, options
+):
+    when_i_attach_staging_token(
+        context, token_type, user_spec, options=options
+    )
+
+
 @when("I attach `{token_type}` {user_spec}")
 def when_i_attach_staging_token(
-    context, token_type, user_spec, verify_return=True
+    context, token_type, user_spec, verify_return=True, options=""
 ):
     token = getattr(context.config, token_type)
     if (
         token_type == "contract_token_staging"
         or token_type == "contract_token_staging_expired"
     ):
-        when_i_run_command(
-            context,
-            "sed -i 's/contracts.can/contracts.staging.can/' {}".format(
-                DEFAULT_CONFIG_FILE
-            ),
-            user_spec,
-        )
-    cmd = "ua attach {}".format(token)
+        change_contract_endpoint_to_staging(context, user_spec)
+    cmd = "pro attach {} {}".format(token, options).strip()
     when_i_run_command(context, cmd, user_spec, verify_return=False)
 
     if verify_return:
@@ -398,7 +558,7 @@ def when_i_create_file_with_content(context, file_path):
     text = context.text.replace('"', '\\"')
     if "<ci-proxy-ip>" in text and "proxy" in context.instances:
         text = text.replace("<ci-proxy-ip>", context.instances["proxy"].ip)
-    cmd = "printf '{}\n' > {}".format(text, file_path)
+    cmd = "printf '{}' > {}".format(text, file_path)
     cmd = 'sh -c "{}"'.format(cmd)
     when_i_run_command(context, cmd, "with sudo")
 
@@ -410,16 +570,43 @@ def when_i_delete_file(context, file_path):
     when_i_run_command(context, cmd, "with sudo")
 
 
-@when("I reboot the `{series}` machine")
-def when_i_reboot_the_machine(context, series):
+@when("I reboot the machine")
+def when_i_reboot_the_machine(context):
     context.instances["uaclient"].restart(wait=True)
 
 
+@when("I reboot the `{machine}` machine")
+def when_i_reboot_the_machine_name(context, machine):
+    context.instances[machine].restart(wait=True)
+
+
 @when("I wait `{seconds}` seconds")
 def when_i_wait(context, seconds):
     time.sleep(int(seconds))
 
 
+@when("I replace `{original}` in `{filename}` with `{new}`")
+def when_i_replace_string_in_file(context, original, filename, new):
+    new = new.replace("\\", r"\\")
+    new = new.replace("/", r"\/")
+    new = new.replace("&", r"\&")
+    when_i_run_command(
+        context,
+        "sed -i 's/{original}/{new}/' {filename}".format(
+            original=original, new=new, filename=filename
+        ),
+        "with sudo",
+    )
+
+
+@when("I replace `{original}` in `{filename}` with token `{token_name}`")
+def when_i_replace_string_in_file_with_token(
+    context, original, filename, token_name
+):
+    token = getattr(context.config, token_name)
+    when_i_replace_string_in_file(context, original, filename, token)
+
+
 @then("I will see the following on stdout")
 def then_i_will_see_on_stdout(context):
     assert_that(context.process.stdout.strip(), equal_to(context.text))
@@ -439,27 +626,6 @@ def then_conditional_stdout_does_not_match_regexp(context, value1, value2):
         then_stream_does_not_match_regexp(context, "stdout")
 
 
-@then("stdout is formatted as `{output_format}` and has keys")
-def then_stdout_is_formatted_and_has_keys(context, output_format):
-    output = context.process.stdout.strip()
-    if output_format == "json":
-        data = json.loads(output)
-    elif output_format == "yaml":
-        data = yaml.safe_load(output)
-
-    keys = set(context.text.split())
-    output_keys = set(data.keys())
-
-    if keys != output_keys:
-        message = """
-        Missing keys in output: {}
-        Extra keys in output: {}
-        """.format(
-            keys - output_keys or "", output_keys - keys or ""
-        )
-        raise AssertionError(message)
-
-
 @then("{stream} does not match regexp")
 def then_stream_does_not_match_regexp(context, stream):
     content = getattr(context.process, stream).strip()
@@ -469,7 +635,16 @@ def then_stream_does_not_match_regexp(context, stream):
 @then("{stream} matches regexp")
 def then_stream_matches_regexp(context, stream):
     content = getattr(context.process, stream).strip()
-    assert_that(content, matches_regexp(context.text))
+    text = context.text
+    if "<ci-proxy-ip>" in text and "proxy" in context.instances:
+        text = text.replace("<ci-proxy-ip>", context.instances["proxy"].ip)
+    assert_that(content, matches_regexp(text))
+
+
+@then("{stream} contains substring")
+def then_stream_contains_substring(context, stream):
+    content = getattr(context.process, stream).strip()
+    assert_that(content, contains_string(context.text))
 
 
 @then("I will see the following on stderr")
@@ -478,27 +653,18 @@ def then_i_will_see_on_stderr(context):
 
 
 @then("I will see the uaclient version on stdout")
-def then_i_will_see_the_uaclient_version_on_stdout(context, feature_str=""):
+def then_i_will_see_the_uaclient_version_on_stdout(context):
     python_import = "from uaclient.version import get_version"
 
     cmd = "python3 -c '{}; print(get_version())'".format(python_import)
 
     actual_version = context.process.stdout.strip()
     when_i_run_command(context, cmd, "as non-root")
-    expected_version = context.process.stdout.strip() + feature_str
+    expected_version = context.process.stdout.strip()
 
     assert_that(expected_version, equal_to(actual_version))
 
 
-@then("I will see the uaclient version on stdout with features `{features}`")
-def then_i_will_see_the_uaclient_version_with_feature_suffix(
-    context, features
-):
-    then_i_will_see_the_uaclient_version_on_stdout(
-        context, feature_str=features
-    )
-
-
 @then("I verify that the `{cmd_name}` command is not found")
 def then_i_should_see_that_the_command_is_not_found(context, cmd_name):
     cmd = "which {} || echo FAILURE".format(cmd_name)
@@ -520,6 +686,29 @@ def then_i_verify_that_running_cmd_with_spec_exits_with_codes(
     assert str(context.process.returncode) in expected_codes
 
 
+@when(
+    "I verify that running attach `{spec}` with json response exits `{exit_codes}`"  # noqa
+)
+def when_i_verify_attach_with_json_response(context, spec, exit_codes):
+    cmd = "pro attach {} --format json".format(context.config.contract_token)
+    then_i_verify_that_running_cmd_with_spec_exits_with_codes(
+        context=context, cmd_name=cmd, spec=spec, exit_codes=exit_codes
+    )
+
+
+@when(
+    "I verify that running attach `{spec}` using expired token with json response fails"  # noqa
+)
+def when_i_verify_attach_expired_token_with_json_response(context, spec):
+    change_contract_endpoint_to_staging(context, user_spec="with sudo")
+    cmd = "pro attach {} --format json".format(
+        context.config.contract_token_staging_expired
+    )
+    then_i_verify_that_running_cmd_with_spec_exits_with_codes(
+        context=context, cmd_name=cmd, spec=spec, exit_codes=ERROR_CODE
+    )
+
+
 @when(
     "I verify that running `{cmd_name}` `{spec}` and stdin `{stdin}` exits `{exit_codes}`"  # noqa
 )
@@ -581,6 +770,16 @@ def verify_installed_package_matches_version_regexp(context, package, regex):
     assert_that(context.process.stdout.strip(), matches_regexp(regex))
 
 
+@then(
+    "I verify that packages `{packages}` installed versions match regexp `{regex}`"  # noqa: E501
+)
+def verify_installed_packages_match_version_regexp(context, packages, regex):
+    for package in packages.split(" "):
+        verify_installed_package_matches_version_regexp(
+            context, package, regex
+        )
+
+
 @then("I verify that `{package}` is installed from apt source `{apt_source}`")
 def verify_package_is_installed_from_apt_source(context, package, apt_source):
     when_i_run_command(
@@ -609,6 +808,18 @@ def verify_package_is_installed_from_apt_source(context, package, apt_source):
     )
 
 
+@then(
+    "I verify that `{packages}` are installed from apt source `{apt_source}`"
+)
+def verify_packages_are_installed_from_apt_source(
+    context, packages, apt_source
+):
+    for package in packages.split(" "):
+        verify_package_is_installed_from_apt_source(
+            context, package, apt_source
+        )
+
+
 @then("I verify that the timer interval for `{job}` is `{interval}`")
 def verify_timer_interval_for_job(context, job, interval):
     when_i_run_command(
@@ -724,7 +935,7 @@ def verify_systemd_timer_ran_or_scheduled(
 def i_save_the_key_value_from_contract(context, key):
     when_i_run_command(
         context,
-        "jq -r '.{}' {}".format(key, DEFAULT_MACHINE_TOKEN_PATH),
+        "jq -r '.{}' {}".format(key, DEFAULT_PRIVATE_MACHINE_TOKEN_PATH),
         "with sudo",
     )
     output = context.process.stdout.strip()
@@ -747,13 +958,21 @@ def _get_saved_attr(context, key):
     return saved_value
 
 
+@then(
+    "I verify that `{key}` value has been updated on the contract on the `{machine}` machine"  # noqa: E501
+)
+def i_verify_that_key_value_has_been_updated_on_machine(context, key, machine):
+    i_verify_that_key_value_has_been_updated(context, key, machine)
+
+
 @then("I verify that `{key}` value has been updated on the contract")
-def i_verify_that_key_value_has_been_updated(context, key):
+def i_verify_that_key_value_has_been_updated(context, key, machine="uaclient"):
     saved_value = _get_saved_attr(context, key)
-    when_i_run_command(
+    when_i_run_command_on_machine(
         context,
-        "jq -r '.{}' {}".format(key, DEFAULT_MACHINE_TOKEN_PATH),
+        "jq -r '.{}' {}".format(key, DEFAULT_PRIVATE_MACHINE_TOKEN_PATH),
         "with sudo",
+        instance_name=machine,
     )
     assert_that(context.process.stdout.strip(), not_(equal_to(saved_value)))
 
@@ -763,7 +982,7 @@ def i_verify_that_key_value_has_not_been_updated(context, key):
     saved_value = _get_saved_attr(context, key)
     when_i_run_command(
         context,
-        "jq -r '.{}' {}".format(key, DEFAULT_MACHINE_TOKEN_PATH),
+        "jq -r '.{}' {}".format(key, DEFAULT_PRIVATE_MACHINE_TOKEN_PATH),
         "with sudo",
     )
     assert_that(context.process.stdout.strip(), equal_to(saved_value))
@@ -779,6 +998,244 @@ def i_restore_the_saved_key_value_on_contract(context, key):
     )
 
 
+@then("stdout is a {output_format} matching the `{schema}` schema")
+def stdout_matches_the_json_schema(context, output_format, schema):
+    if output_format == "json":
+        instance = json.loads(context.process.stdout.strip())
+    elif output_format == "yaml":
+        instance = yaml.load(
+            context.process.stdout.strip(), SafeLoaderWithoutDatetime
+        )
+    with open("features/schemas/{}.json".format(schema), "r") as schema_file:
+        jsonschema.validate(instance=instance, schema=json.load(schema_file))
+
+
+@then("the {output_format} API response data matches the `{schema}` schema")
+def api_response_matches_schema(context, output_format, schema):
+    if output_format == "json":
+        instance = json.loads(context.process.stdout.strip())
+    elif output_format == "yaml":
+        instance = yaml.load(
+            context.process.stdout.strip(), SafeLoaderWithoutDatetime
+        )
+    with open("features/schemas/{}.json".format(schema), "r") as schema_file:
+        jsonschema.validate(
+            instance=instance.get("data", {}).get("attributes"),
+            schema=json.load(schema_file),
+        )
+
+
+@then("`{file_name}` is not present in any docker image layer")
+def file_is_not_present_in_any_docker_image_layer(context, file_name):
+    when_i_run_command(
+        context,
+        "find /var/lib/docker/overlay2 -name {}".format(file_name),
+        "with sudo",
+    )
+    results = context.process.stdout.strip()
+    if results:
+        raise AssertionError(
+            'found "{}"'.format(", ".join(results.split("\n")))
+        )
+
+
+# This defines "not significantly larger" as "less than 2MB larger"
+@then(
+    "docker image `{name}` is not significantly larger than `ubuntu:{series}` with `{package}` installed"  # noqa: E501
+)
+def docker_image_is_not_larger(context, name, series, package):
+    base_image_name = "ubuntu:{}".format(series)
+    base_upgraded_image_name = "{}-with-test-package".format(series)
+
+    # We need to compare against the base image after apt upgrade
+    # and package install
+    dockerfile = """\
+    FROM {}
+    RUN apt-get update \\
+      && apt-get install -y {} \\
+      && rm -rf /var/lib/apt/lists/*
+    """.format(
+        base_image_name, package
+    )
+    context.text = dockerfile
+    when_i_create_file_with_content(context, "Dockerfile.base")
+    when_i_run_command(
+        context,
+        "docker build . -f Dockerfile.base -t {}".format(
+            base_upgraded_image_name
+        ),
+        "with sudo",
+    )
+
+    # find image sizes
+    when_i_run_shell_command(
+        context, "docker inspect {} | jq .[0].Size".format(name), "with sudo"
+    )
+    custom_image_size = int(context.process.stdout.strip())
+    when_i_run_shell_command(
+        context,
+        "docker inspect {} | jq .[0].Size".format(base_upgraded_image_name),
+        "with sudo",
+    )
+    base_image_size = int(context.process.stdout.strip())
+
+    # Get pro test deb size
+    when_i_run_command(context, "du ubuntu-advantage-tools.deb", "with sudo")
+    # Example out: "1234\tubuntu-advantage-tools.deb"
+    ua_test_deb_size = (
+        int(context.process.stdout.strip().split("\t")[0]) * 1024
+    )  # KB -> B
+
+    # Give us some space for bloat we don't control: 2MB -> B
+    extra_space = 2 * 1024 * 1024
+
+    if custom_image_size > (base_image_size + ua_test_deb_size + extra_space):
+        raise AssertionError(
+            "Custom image size ({}) is over 2MB greater than the base image"
+            " size ({}) + pro test deb size ({})".format(
+                custom_image_size, base_image_size, ua_test_deb_size
+            )
+        )
+    logging.debug(
+        "custom image size ({})\n"
+        "base image size ({})\n"
+        "pro test deb size ({})".format(
+            custom_image_size, base_image_size, ua_test_deb_size
+        )
+    )
+
+
+@then(
+    "on `{release}`, systemd status output says memory usage is less than `{mb_limit}` MB"  # noqa
+)
+def systemd_memory_usage_less_than(context, release, mb_limit):
+    curr_release = context.active_outline["release"]
+    if release != curr_release:
+        logging.debug("Skipping for {}".format(curr_release))
+        return
+    match = re.search(r"Memory: (.*)M", context.process.stdout.strip())
+    if match is None:
+        raise AssertionError(
+            "Memory usage not present in current process stdout"
+        )
+    mb_used = float(match.group(1))
+    logging.debug("Found {}M".format(mb_used))
+
+    mb_limit_float = float(mb_limit)
+    if mb_used > mb_limit_float:
+        raise AssertionError(
+            "Using more memory than expected ({}M)".format(mb_used)
+        )
+
+
+@when(
+    "I prepare the local PPAs to upgrade from `{release}` to `{next_release}`"
+)
+def when_i_create_local_ppas(context, release, next_release):
+    if context.config.install_from is not InstallationSource.LOCAL:
+        return
+
+    # We need Kinetic or greater to support zstd when creating the PPAs
+    launch_machine(context, "kinetic", "ppa")
+    when_i_run_command_on_machine(
+        context, "apt-get update", "with sudo", "ppa"
+    )
+    when_i_run_command_on_machine(
+        context, "apt-get install -y aptly", "with sudo", "ppa"
+    )
+    create_local_ppa(context, release)
+    create_local_ppa(context, next_release)
+    repo_line = "deb [trusted=yes] http://{}:8080 {} main".format(
+        context.instances["ppa"].ip, release
+    )
+    repo_file = "/etc/apt/sources.list.d/local-ua.list"
+    when_i_run_shell_command(
+        context, "printf '{}\n' > {}".format(repo_line, repo_file), "with sudo"
+    )
+    when_i_run_command_on_machine(
+        context,
+        "sh -c 'nohup aptly serve > /dev/null 2>&1 &'",
+        "with sudo",
+        "ppa",
+    )
+
+
+@when("I verify root and non-root `{cmd}` calls have the same output")
+def root_vs_nonroot_cmd_comparison(context, cmd):
+    when_i_run_command(context, cmd, "with sudo")
+    root_status_stdout = context.process.stdout.strip()
+    root_status_stderr = context.process.stderr.strip()
+
+    when_i_run_command(context, cmd, "as non-root")
+    nonroot_status_stdout = context.process.stdout.strip()
+    nonroot_status_stderr = context.process.stderr.strip()
+
+    assert_that(root_status_stdout, nonroot_status_stdout)
+    assert root_status_stderr == nonroot_status_stderr
+
+
+@when("I install third-party / unknown packages in the machine")
+def when_i_install_packages(context):
+    # The `code` deb package sets up an apt remote for updates,
+    # and is then listed as third-party.
+    # https://code.visualstudio.com/download
+
+    # The `gh` deb package is just installed locally,
+    # and is then listed as unknown
+    # https://github.com/cli/cli/releases
+    when_i_run_command(
+        context,
+        (
+            "curl -L "
+            "https://az764295.vo.msecnd.net/stable/"
+            "e4503b30fc78200f846c62cf8091b76ff5547662/"
+            "code_1.70.2-1660629410_amd64.deb "
+            "-o /tmp/code.deb"
+        ),
+        "with sudo",
+    )
+    when_i_run_command(
+        context,
+        (
+            "curl -L "
+            "https://github.com/cli/cli/releases/download/"
+            "v2.14.4/gh_2.14.4_linux_amd64.deb "
+            "-o /tmp/gh.deb"
+        ),
+        "with sudo",
+    )
+    when_i_run_command(
+        context, "apt-get install -y /tmp/code.deb", "with sudo"
+    )
+    when_i_run_command(context, "apt-get install -y /tmp/gh.deb", "with sudo")
+    when_i_run_command(context, "apt-get update", "with sudo")
+
+
+def create_local_ppa(context, release):
+    when_i_run_command_on_machine(
+        context,
+        "aptly repo create -distribution {} repo-{}".format(release, release),
+        "with sudo",
+        "ppa",
+    )
+    debs = build_debs_from_sbuild(context, release)
+    for deb in debs:
+        deb_destination = "/tmp/" + deb.split("/")[-1]
+        context.instances["ppa"].push_file(deb, deb_destination)
+        when_i_run_command_on_machine(
+            context,
+            "aptly repo add repo-{} {}".format(release, deb_destination),
+            "with sudo",
+            "ppa",
+        )
+    when_i_run_command_on_machine(
+        context,
+        "aptly publish repo -skip-signing repo-{}".format(release),
+        "with sudo",
+        "ppa",
+    )
+
+
 def get_command_prefix_for_user_spec(user_spec):
     prefix = []
     if user_spec == "with sudo":
