@@ -1,14 +1,12 @@
 import hashlib
 import logging
-import multiprocessing
 import os
 import shutil
 import subprocess
 import tempfile
 import textwrap
-import time
-from contextlib import contextmanager
-from typing import Iterable, List
+from enum import Enum
+from typing import Iterable, List, Optional
 
 import yaml
 
@@ -16,7 +14,6 @@ LXC_PROPERTY_MAP = {
     "image": {"series": "properties.release", "machine_type": "Type"},
     "container": {"series": "image.release", "machine_type": "image.type"},
 }
-SLOW_CMDS = ["do-release-upgrade"]  # Commands which will emit dots on travis
 UA_TMP_DIR = os.path.join(tempfile.gettempdir(), "uaclient-behave")
 SOURCE_PR_TGZ = os.path.join(UA_TMP_DIR, "pr_source.tar.gz")
 SOURCE_PR_UNTAR_DIR = os.path.join(UA_TMP_DIR, "behave-ua-src")
@@ -43,6 +40,16 @@ BUILD_FROM_TGZ = textwrap.dedent(
 )
 
 
+class InstallationSource(Enum):
+    ARCHIVE = "archive"
+    LOCAL = "local"
+    DAILY = "daily"
+    STAGING = "staging"
+    STABLE = "stable"
+    PROPOSED = "proposed"
+    CUSTOM = "custom"
+
+
 def lxc_get_property(name: str, property_name: str, image: bool = False):
     """Check series name of either an image or a container.
 
@@ -95,6 +102,8 @@ def repo_state_hash(
         "features",
         "sru",
         "tools",
+        "docs",
+        "dev-docs",
     )
 ):
     """
@@ -153,15 +162,13 @@ def repo_state_hash(
     return hashlib.md5(output_to_hash).hexdigest()
 
 
-def build_debs(series: str, cache_source: bool = False) -> List[str]:
+def build_debs(series: str, chroot: Optional[str] = None) -> List[str]:
     """
     Build the package through sbuild and store the debs into
     output_deb_dir
 
     :param series: The target series to build the package for
     :param output_deb_dir: the target directory in which to copy deb artifacts
-    :param cache_source: If False, we will always rebuild the source environemt
-                         for sbuild
 
     :return: A list of file paths to debs created by the build.
     """
@@ -185,45 +192,42 @@ def build_debs(series: str, cache_source: bool = False) -> List[str]:
         return [tools_deb_cache_path, pro_deb_cache_path]
 
     logging.info("--- Creating: {}".format(SOURCE_PR_TGZ))
-    if not os.path.exists(os.path.dirname(SOURCE_PR_TGZ)):
-        os.makedirs(os.path.dirname(SOURCE_PR_TGZ))
 
-    if not os.path.exists(SOURCE_PR_TGZ) or not cache_source:
-        cwd = os.getcwd()
-        os.chdir("..")
-        subprocess.run(
-            [
-                "tar",
-                "-zcf",
-                SOURCE_PR_TGZ,
-                "--exclude-vcs",
-                "--exclude-vcs-ignores",
-                os.path.basename(cwd),
-            ],
-            check=True,
-        )
-        os.chdir(cwd)
+    cwd = os.getcwd()
+    os.chdir("..")
+    subprocess.run(
+        [
+            "tar",
+            "-zcf",
+            SOURCE_PR_TGZ,
+            "--exclude-vcs",
+            "--exclude-vcs-ignores",
+            os.path.basename(cwd),
+        ],
+        check=True,
+    )
+    os.chdir(cwd)
 
     logging.info("--- Creating: {}".format(SOURCE_PR_UNTAR_DIR))
-    if not os.path.exists(SOURCE_PR_UNTAR_DIR) or not cache_source:
-        # Delete cached folder for ua code
-        if os.path.exists(SOURCE_PR_UNTAR_DIR):
-            shutil.rmtree(SOURCE_PR_UNTAR_DIR)
 
-        os.makedirs(SOURCE_PR_UNTAR_DIR)
-        subprocess.run(
-            [
-                "tar",
-                "-xvf",
-                SOURCE_PR_TGZ,
-                "-C",
-                SOURCE_PR_UNTAR_DIR,
-                "--strip-components=1",
-            ],
-            stdout=subprocess.DEVNULL,
-            stderr=subprocess.DEVNULL,
-            check=True,
-        )
+    # Delete cached folder for pro code
+    if os.path.exists(SOURCE_PR_UNTAR_DIR):
+        shutil.rmtree(SOURCE_PR_UNTAR_DIR)
+
+    os.makedirs(SOURCE_PR_UNTAR_DIR)
+    subprocess.run(
+        [
+            "tar",
+            "-xvf",
+            SOURCE_PR_TGZ,
+            "-C",
+            SOURCE_PR_UNTAR_DIR,
+            "--strip-components=1",
+        ],
+        stdout=subprocess.DEVNULL,
+        stderr=subprocess.DEVNULL,
+        check=True,
+    )
 
     if os.path.exists(SBUILD_DIR):
         shutil.rmtree(SBUILD_DIR)
@@ -231,19 +235,34 @@ def build_debs(series: str, cache_source: bool = False) -> List[str]:
 
     curr_dir = os.getcwd()
     os.chdir(SBUILD_DIR)
-    logging.info("--- Running sbuild")
+
+    sbuild_cmd = [
+        "sbuild",
+        "--no-run-lintian",
+        "--resolve-alternatives",
+        "--no-clean-source",
+        "--arch",
+        "amd64",
+        "-d",
+        series,
+        SOURCE_PR_UNTAR_DIR,
+    ]
+    if chroot is not None:
+        sbuild_cmd += ["--chroot", chroot]
+    else:
+        # use ua-series-arch chroot if present
+        ua_chroot = "ua-{}-amd64".format(series)
+        proc = subprocess.run(
+            ["schroot", "--info", "--chroot", ua_chroot],
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.DEVNULL,
+        )
+        if proc.returncode == 0:
+            sbuild_cmd += ["--chroot", ua_chroot]
+
+    logging.info('--- Running "{}"'.format(" ".join(sbuild_cmd)))
     subprocess.run(
-        [
-            "sbuild",
-            "--no-run-lintian",
-            "--resolve-alternatives",
-            "--no-clean-source",
-            "--arch",
-            "amd64",
-            "-d",
-            series,
-            SOURCE_PR_UNTAR_DIR,
-        ],
+        sbuild_cmd,
         stdout=subprocess.DEVNULL,
         stderr=subprocess.DEVNULL,
         check=True,
@@ -264,47 +283,29 @@ def build_debs(series: str, cache_source: bool = False) -> List[str]:
     return [tools_deb_cache_path, pro_deb_cache_path]
 
 
-# Support for python 3.6 or earlier
-@contextmanager
-def nullcontext(enter_result=None):
-    yield enter_result
+class SafeLoaderWithoutDatetime(yaml.SafeLoader):
+    yaml_implicit_resolvers = {
+        k: [r for r in v if r[0] != "tag:yaml.org,2002:timestamp"]
+        for k, v in yaml.SafeLoader.yaml_implicit_resolvers.items()
+    }
 
 
-def spinning_cursor():
-    while True:
-        for cursor in "|/-\\":
-            yield cursor
+def cleanup_instance(context, instance_name):
+    def _cleanup_instance():
+        if not context.config.destroy_instances:
+            logging.info(
+                "--- Leaving instance running: {}".format(
+                    context.instances[instance_name].name
+                )
+            )
+            return
+        try:
+            context.instances[instance_name].delete(wait=False)
+        except RuntimeError as e:
+            logging.error(
+                "Failed to delete instance: {}\n{}".format(
+                    context.instances[instance_name].name, str(e)
+                )
+            )
 
-
-@contextmanager
-def emit_spinner_on_travis(msg: str = " "):
-    """
-    A context manager that emits a spinner updating 5 seconds if running on
-    Travis.
-
-    Travis will kill jobs that don't emit output for a certain amount of time.
-    This context manager spins up a background process which will emit a char
-    to stdout every 10 seconds to avoid being killed.
-
-    It should be wrapped selectively around operations that are known to take a
-    long time
-    """
-    if os.environ.get("TRAVIS") != "true":
-        # If we aren't on Travis, don't do anything.
-        yield
-        return
-
-    def emit_spinner():
-        print(msg, end="", flush=True)
-        spinner = spinning_cursor()
-        while True:
-            time.sleep(5)
-            print("\b%s" % next(spinner), end="", flush=True)
-
-    dot_process = multiprocessing.Process(target=emit_spinner)
-    dot_process.start()
-    try:
-        yield
-    finally:
-        print()
-        dot_process.terminate()
+    return _cleanup_instance
