@@ -1,12 +1,21 @@
 """Tests related to uaclient.entitlement.base module."""
 import logging
-from typing import Dict, Optional, Tuple
+from typing import Any, Dict, Optional, Tuple
 
 import mock
 import pytest
 
-from uaclient import config, status, util
+from uaclient import messages, util
 from uaclient.entitlements import base
+from uaclient.entitlements.entitlement_status import (
+    ApplicabilityStatus,
+    ApplicationStatus,
+    CanDisableFailure,
+    CanDisableFailureReason,
+    CanEnableFailure,
+    CanEnableFailureReason,
+    UserFacingStatus,
+)
 from uaclient.status import ContractStatus
 
 
@@ -21,19 +30,29 @@ class ConcreteTestEntitlement(base.UAEntitlement):
         cfg=None,
         disable=None,
         enable=None,
-        applicability_status=None,
-        application_status=None,
+        applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+        application_status=(ApplicationStatus.DISABLED, ""),
         allow_beta=False,
+        supports_access_only=False,
+        access_only=False,
+        dependent_services=None,
+        required_services=None,
+        blocking_incompatible_services=None,
+        **kwargs
     ):
-        super().__init__(cfg, allow_beta=allow_beta)
+        super().__init__(cfg, allow_beta=allow_beta, access_only=access_only)
+        self.supports_access_only = supports_access_only
         self._disable = disable
         self._enable = enable
         self._applicability_status = applicability_status
         self._application_status = application_status
+        self._dependent_services = dependent_services
+        self._required_services = required_services
+        self._blocking_incompatible_services = blocking_incompatible_services
 
     def _perform_disable(self, **kwargs):
         self._application_status = (
-            status.ApplicationStatus.DISABLED,
+            ApplicationStatus.DISABLED,
             "disable() called",
         )
         return self._disable
@@ -47,20 +66,36 @@ class ConcreteTestEntitlement(base.UAEntitlement):
     def application_status(self):
         return self._application_status
 
+    def blocking_incompatible_services(self):
+        if self._blocking_incompatible_services is not None:
+            return self._blocking_incompatible_services
+        else:
+            return super().blocking_incompatible_services()
+
 
 @pytest.fixture
-def concrete_entitlement_factory(tmpdir):
+def concrete_entitlement_factory(FakeConfig):
     def factory(
         *,
         entitled: bool,
-        applicability_status: Tuple[status.ApplicabilityStatus, str] = None,
-        application_status: Tuple[status.ApplicationStatus, str] = None,
+        applicability_status: Tuple[ApplicabilityStatus, str] = (
+            ApplicabilityStatus.APPLICABLE,
+            "",
+        ),
+        application_status: Tuple[ApplicationStatus, str] = (
+            ApplicationStatus.DISABLED,
+            "",
+        ),
         feature_overrides: Optional[Dict[str, str]] = None,
         allow_beta: bool = False,
+        supports_access_only: bool = False,
+        access_only: bool = False,
         enable: bool = False,
-        disable: bool = False
+        disable: bool = False,
+        dependent_services: Tuple[Any, ...] = None,
+        required_services: Tuple[Any, ...] = None
     ) -> ConcreteTestEntitlement:
-        cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
+        cfg = FakeConfig()
         machineToken = {
             "machineToken": "blah",
             "machineTokenInfo": {
@@ -74,7 +109,7 @@ def concrete_entitlement_factory(tmpdir):
                 }
             },
         }
-        cfg.write_cache("machine-token", machineToken)
+        cfg.machine_token_file.write(machineToken)
 
         if feature_overrides:
             cfg.cfg.update({"features": feature_overrides})
@@ -84,8 +119,12 @@ def concrete_entitlement_factory(tmpdir):
             applicability_status=applicability_status,
             application_status=application_status,
             allow_beta=allow_beta,
+            supports_access_only=supports_access_only,
+            access_only=access_only,
             enable=enable,
             disable=disable,
+            dependent_services=dependent_services,
+            required_services=required_services,
         )
 
     return factory
@@ -108,29 +147,75 @@ class TestUaEntitlement:
         entitlement = ConcreteTestEntitlement()
         assert "/var/lib/ubuntu-advantage" == entitlement.cfg.data_dir
 
-    def test_init_accepts_a_uaconfig(self):
+    def test_init_accepts_a_uaconfig(self, FakeConfig):
         """An instance of UAConfig can be passed to UAEntitlement."""
-        cfg = config.UAConfig(cfg={"data_dir": "/some/path"})
+        cfg = FakeConfig(cfg_overrides={"data_dir": "/some/path"})
         entitlement = ConcreteTestEntitlement(cfg)
         assert "/some/path" == entitlement.cfg.data_dir
 
     def test_can_disable_false_on_entitlement_inactive(
-        self, capsys, concrete_entitlement_factory
+        self, concrete_entitlement_factory
     ):
         """When  status is INACTIVE, can_disable returns False."""
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
 
-        assert not entitlement.can_disable()
+        ret, fail = entitlement.can_disable()
+        assert not ret
 
-        expected_stdout = (
+        expected_msg = (
             "Test Concrete Entitlement is not currently enabled\n"
-            "See: sudo ua status\n"
+            "See: sudo pro status"
         )
-        stdout, _ = capsys.readouterr()
-        assert expected_stdout == stdout
+        assert expected_msg == fail.message.msg
+
+    def test_can_disable_false_on_dependent_service(
+        self, concrete_entitlement_factory
+    ):
+        """When  status is INACTIVE, can_disable returns False."""
+        m_ent_cls = mock.Mock()
+        type(m_ent_cls).name = mock.PropertyMock(return_value="test")
+        m_ent_obj = m_ent_cls.return_value
+        m_ent_obj.application_status.return_value = (
+            ApplicationStatus.ENABLED,
+            None,
+        )
+
+        entitlement = concrete_entitlement_factory(
+            entitled=True,
+            application_status=(ApplicationStatus.ENABLED, ""),
+            dependent_services=(m_ent_cls,),
+        )
+
+        ret, fail = entitlement.can_disable()
+        assert not ret
+        assert fail.reason == CanDisableFailureReason.ACTIVE_DEPENDENT_SERVICES
+        assert fail.message is None
+
+    @mock.patch("uaclient.entitlements.entitlement_factory")
+    def test_can_disable_when_ignoring_dependent_service(
+        self, m_ent_factory, concrete_entitlement_factory
+    ):
+        """When  status is INACTIVE, can_disable returns False."""
+        entitlement = concrete_entitlement_factory(
+            entitled=True,
+            application_status=(ApplicationStatus.ENABLED, ""),
+            dependent_services=("test",),
+        )
+
+        m_ent_cls = mock.Mock()
+        m_ent_obj = m_ent_cls.return_value
+        m_ent_obj.application_status.return_value = (
+            ApplicationStatus.ENABLED,
+            None,
+        )
+        m_ent_factory.return_value = m_ent_cls
+
+        ret, fail = entitlement.can_disable(ignore_dependent_services=True)
+        assert ret is True
+        assert fail is None
 
     def test_can_disable_true_on_entitlement_active(
         self, capsys, concrete_entitlement_factory
@@ -138,7 +223,7 @@ class TestUaEntitlement:
         """When entitlement is ENABLED, can_disable returns True."""
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            application_status=(status.ApplicationStatus.ENABLED, ""),
+            application_status=(ApplicationStatus.ENABLED, ""),
         )
 
         assert entitlement.can_disable()
@@ -154,9 +239,34 @@ class TestUaEntitlement:
 
         can_enable, reason = entitlement.can_enable()
         assert not can_enable
-        assert reason.reason == status.CanEnableFailureReason.NOT_ENTITLED
-        assert reason.message == status.MESSAGE_UNENTITLED_TMPL.format(
-            title=ConcreteTestEntitlement.title
+        assert reason.reason == CanEnableFailureReason.NOT_ENTITLED
+        assert (
+            reason.message.msg
+            == messages.UNENTITLED.format(
+                title=ConcreteTestEntitlement.title
+            ).msg
+        )
+
+    def test_can_enable_false_on_access_only_not_supported(
+        self, concrete_entitlement_factory
+    ):
+        """When entitlement contract is not enabled, can_enable is False."""
+        entitlement = concrete_entitlement_factory(
+            entitled=True,
+            supports_access_only=False,
+            access_only=True,
+        )
+
+        can_enable, reason = entitlement.can_enable()
+        assert not can_enable
+        assert (
+            reason.reason == CanEnableFailureReason.ACCESS_ONLY_NOT_SUPPORTED
+        )
+        assert (
+            reason.message.msg
+            == messages.ENABLE_ACCESS_ONLY_NOT_SUPPORTED.format(
+                title=ConcreteTestEntitlement.title
+            ).msg
         )
 
     @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
@@ -186,16 +296,19 @@ class TestUaEntitlement:
         self, concrete_entitlement_factory
     ):
         """When entitlement is ENABLED, can_enable returns False."""
-        application_status = status.ApplicationStatus.ENABLED
+        application_status = ApplicationStatus.ENABLED
         entitlement = concrete_entitlement_factory(
             entitled=True, application_status=(application_status, "")
         )
 
         can_enable, reason = entitlement.can_enable()
         assert not can_enable
-        assert reason.reason == status.CanEnableFailureReason.ALREADY_ENABLED
-        assert reason.message == status.MESSAGE_ALREADY_ENABLED_TMPL.format(
-            title=ConcreteTestEntitlement.title
+        assert reason.reason == CanEnableFailureReason.ALREADY_ENABLED
+        assert (
+            reason.message.msg
+            == messages.ALREADY_ENABLED.format(
+                title=ConcreteTestEntitlement.title
+            ).msg
         )
 
     def test_can_enable_false_on_entitlement_inapplicable(
@@ -205,15 +318,15 @@ class TestUaEntitlement:
         entitlement = concrete_entitlement_factory(
             entitled=True,
             applicability_status=(
-                status.ApplicabilityStatus.INAPPLICABLE,
+                ApplicabilityStatus.INAPPLICABLE,
                 "msg",
             ),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
 
         can_enable, reason = entitlement.can_enable()
         assert not can_enable
-        assert reason.reason == status.CanEnableFailureReason.INAPPLICABLE
+        assert reason.reason == CanEnableFailureReason.INAPPLICABLE
         assert reason.message == "msg"
 
     def test_can_enable_true_on_entitlement_inactive(
@@ -222,8 +335,8 @@ class TestUaEntitlement:
         """When an entitlement is applicable and disabled, we can_enable"""
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
 
         can_enable, reason = entitlement.can_enable()
@@ -240,8 +353,8 @@ class TestUaEntitlement:
         feature_overrides = {"allow_beta": allow_beta_cfg}
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
             feature_overrides=feature_overrides,
             allow_beta=allow_beta,
         )
@@ -253,7 +366,7 @@ class TestUaEntitlement:
             assert reason is None
         else:
             assert not can_enable
-            assert reason.reason == status.CanEnableFailureReason.IS_BETA
+            assert reason.reason == CanEnableFailureReason.IS_BETA
             assert reason.message is None
 
     def test_contract_status_entitled(self, concrete_entitlement_factory):
@@ -276,8 +389,8 @@ class TestUaEntitlement:
         """When orig_acccess dict is empty perform no work."""
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
         with mock.patch.object(entitlement, "can_disable") as m_can_disable:
             entitlement.process_contract_deltas(orig_access, delta)
@@ -288,62 +401,51 @@ class TestUaEntitlement:
     ):
         base_ent = concrete_entitlement_factory(
             entitled=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
-        base_ent._incompatible_services = ["test"]
 
         m_entitlement_cls = mock.MagicMock()
         m_entitlement_obj = m_entitlement_cls.return_value
         m_entitlement_obj.application_status.return_value = [
-            status.ApplicationStatus.ENABLED,
+            ApplicationStatus.ENABLED,
             "",
         ]
-        type(m_entitlement_obj).title = mock.PropertyMock(return_value="test")
+        base_ent._incompatible_services = (
+            base.IncompatibleService(
+                m_entitlement_cls, messages.NamedMessage("test", "test")
+            ),
+        )
 
-        with mock.patch.object(
-            base_ent, "is_access_expired", return_value=False
-        ):
-            with mock.patch(
-                "uaclient.entitlements.entitlement_factory",
-                return_value=m_entitlement_cls,
-            ):
-                ret, reason = base_ent.can_enable()
+        ret, reason = base_ent.can_enable()
 
         assert ret is False
-        assert (
-            reason.reason == status.CanEnableFailureReason.INCOMPATIBLE_SERVICE
-        )
+        assert reason.reason == CanEnableFailureReason.INCOMPATIBLE_SERVICE
         assert reason.message is None
 
     def test_can_enable_when_required_service_found(
         self, concrete_entitlement_factory
     ):
-        base_ent = concrete_entitlement_factory(
-            entitled=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
-        )
-        base_ent._required_services = ["test"]
-
-        m_entitlement_cls = mock.MagicMock()
-        m_entitlement_obj = m_entitlement_cls.return_value
-        m_entitlement_obj.application_status.return_value = [
-            status.ApplicationStatus.DISABLED,
+        m_ent_cls = mock.MagicMock()
+        type(m_ent_cls).name = mock.PropertyMock(return_value="test")
+        m_ent_obj = m_ent_cls.return_value
+        m_ent_obj.application_status.return_value = [
+            ApplicationStatus.DISABLED,
             "",
         ]
-        type(m_entitlement_obj).title = mock.PropertyMock(return_value="test")
+        type(m_ent_obj).title = mock.PropertyMock(return_value="test")
+        base_ent = concrete_entitlement_factory(
+            entitled=True,
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
+            required_services=(m_ent_cls,),
+        )
 
-        with mock.patch(
-            "uaclient.entitlements.entitlement_factory",
-            return_value=m_entitlement_cls,
-        ):
-            ret, reason = base_ent.can_enable()
+        ret, reason = base_ent.can_enable()
 
         assert ret is False
         assert (
-            reason.reason
-            == status.CanEnableFailureReason.INACTIVE_REQUIRED_SERVICES
+            reason.reason == CanEnableFailureReason.INACTIVE_REQUIRED_SERVICES
         )
         assert reason.message is None
 
@@ -366,34 +468,30 @@ class TestUaEntitlement:
         base_ent = concrete_entitlement_factory(
             entitled=True,
             enable=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
-        base_ent._incompatible_services = ["test"]
 
         m_entitlement_cls = mock.MagicMock()
         m_entitlement_obj = m_entitlement_cls.return_value
         m_entitlement_obj.application_status.return_value = [
-            status.ApplicationStatus.ENABLED,
+            ApplicationStatus.ENABLED,
             "",
         ]
-        type(m_entitlement_obj).title = mock.PropertyMock(return_value="test")
+        base_ent._incompatible_services = (
+            base.IncompatibleService(
+                m_entitlement_cls, messages.NamedMessage("test", "test")
+            ),
+        )
 
-        with mock.patch.object(
-            base_ent, "is_access_expired", return_value=False
-        ):
-            with mock.patch(
-                "uaclient.entitlements.entitlement_factory",
-                return_value=m_entitlement_cls,
-            ):
-                ret, reason = base_ent.enable()
+        ret, reason = base_ent.enable()
 
         expected_prompt_call = 1
         if block_disable_on_enable:
             expected_prompt_call = 0
 
         expected_ret = False
-        expected_reason = status.CanEnableFailureReason.INCOMPATIBLE_SERVICE
+        expected_reason = CanEnableFailureReason.INCOMPATIBLE_SERVICE
         if assume_yes and not block_disable_on_enable:
             expected_ret = True
             expected_reason = None
@@ -414,35 +512,30 @@ class TestUaEntitlement:
         self, m_prompt, assume_yes, concrete_entitlement_factory
     ):
         m_prompt.return_value = assume_yes
+
+        m_ent_cls = mock.MagicMock()
+        type(m_ent_cls).name = mock.PropertyMock(return_value="test")
+        m_ent_obj = m_ent_cls.return_value
+        m_ent_obj.application_status.return_value = [
+            ApplicationStatus.DISABLED,
+            "",
+        ]
+        m_ent_obj.enable.return_value = (True, "")
+        type(m_ent_obj).title = mock.PropertyMock(return_value="test")
+
         base_ent = concrete_entitlement_factory(
             entitled=True,
             enable=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
+            required_services=(m_ent_cls,),
         )
-        base_ent._required_services = ("test",)
 
-        m_entitlement_cls = mock.MagicMock()
-        m_entitlement_obj = m_entitlement_cls.return_value
-        m_entitlement_obj.application_status.return_value = [
-            status.ApplicationStatus.DISABLED,
-            "",
-        ]
-        m_entitlement_obj.enable.return_value = (True, "")
-        type(m_entitlement_obj).title = mock.PropertyMock(return_value="test")
-
-        with mock.patch(
-            "uaclient.entitlements.entitlement_factory",
-            return_value=m_entitlement_cls,
-        ):
-            ret, reason = base_ent.enable()
+        ret, reason = base_ent.enable()
 
         expected_prompt_call = 1
-
         expected_ret = False
-        expected_reason = (
-            status.CanEnableFailureReason.INACTIVE_REQUIRED_SERVICES
-        )
+        expected_reason = CanEnableFailureReason.INACTIVE_REQUIRED_SERVICES
         if assume_yes:
             expected_ret = True
             expected_reason = None
@@ -454,48 +547,44 @@ class TestUaEntitlement:
         else:
             assert reason.reason == expected_reason
         assert m_prompt.call_count == expected_prompt_call
-        assert m_entitlement_obj.enable.call_count == expected_enable_call
+        assert m_ent_obj.enable.call_count == expected_enable_call
 
     @pytest.mark.parametrize(
         "can_enable_fail,handle_incompat_calls,enable_req_calls",
         [
             (
-                status.CanEnableFailure(
-                    status.CanEnableFailureReason.NOT_ENTITLED, message="msg"
+                CanEnableFailure(
+                    CanEnableFailureReason.NOT_ENTITLED, message="msg"
                 ),
                 0,
                 0,
             ),
             (
-                status.CanEnableFailure(
-                    status.CanEnableFailureReason.ALREADY_ENABLED,
+                CanEnableFailure(
+                    CanEnableFailureReason.ALREADY_ENABLED,
                     message="msg",
                 ),
                 0,
                 0,
             ),
             (
-                status.CanEnableFailure(status.CanEnableFailureReason.IS_BETA),
+                CanEnableFailure(CanEnableFailureReason.IS_BETA),
                 0,
                 0,
             ),
             (
-                status.CanEnableFailure(
-                    status.CanEnableFailureReason.INAPPLICABLE, "msg"
-                ),
+                CanEnableFailure(CanEnableFailureReason.INAPPLICABLE, "msg"),
                 0,
                 0,
             ),
             (
-                status.CanEnableFailure(
-                    status.CanEnableFailureReason.INCOMPATIBLE_SERVICE
-                ),
+                CanEnableFailure(CanEnableFailureReason.INCOMPATIBLE_SERVICE),
                 1,
                 0,
             ),
             (
-                status.CanEnableFailure(
-                    status.CanEnableFailureReason.INACTIVE_REQUIRED_SERVICES
+                CanEnableFailure(
+                    CanEnableFailureReason.INACTIVE_REQUIRED_SERVICES
                 ),
                 0,
                 1,
@@ -504,7 +593,7 @@ class TestUaEntitlement:
     )
     @mock.patch(
         "uaclient.entitlements.base.UAEntitlement._enable_required_services",
-        return_value=False,
+        return_value=(False, "required error msg"),
     )
     @mock.patch(
         "uaclient.entitlements.base.UAEntitlement.handle_incompatible_services",  # noqa: E501
@@ -523,6 +612,7 @@ class TestUaEntitlement:
     ):
         """When can_enable returns False enable returns False."""
         m_can_enable.return_value = (False, can_enable_fail)
+        m_handle_incompat.return_value = (False, None)
         entitlement = concrete_entitlement_factory(entitled=True)
         entitlement._perform_enable = mock.Mock()
 
@@ -533,6 +623,63 @@ class TestUaEntitlement:
         assert enable_req_calls == m_enable_required.call_count
         assert 0 == entitlement._perform_enable.call_count
 
+        if enable_req_calls:
+            assert can_enable_fail.message == "required error msg"
+
+    @pytest.mark.parametrize("enable_fail_message", (("not entitled"), (None)))
+    @mock.patch("uaclient.util.handle_message_operations")
+    @mock.patch("uaclient.util.prompt_for_confirmation")
+    def test_enable_false_when_fails_to_enable_required_service(
+        self,
+        m_handle_msg,
+        m_prompt_for_confirmation,
+        enable_fail_message,
+        concrete_entitlement_factory,
+    ):
+        m_handle_msg.return_value = True
+
+        fail_reason = CanEnableFailure(
+            CanEnableFailureReason.INACTIVE_REQUIRED_SERVICES
+        )
+
+        if enable_fail_message:
+            msg = messages.NamedMessage("test-code", enable_fail_message)
+        else:
+            msg = None
+
+        enable_fail_reason = CanEnableFailure(
+            CanEnableFailureReason.NOT_ENTITLED, message=msg
+        )
+
+        m_ent_cls = mock.Mock()
+        type(m_ent_cls).name = mock.PropertyMock(return_value="Test")
+        m_ent_obj = m_ent_cls.return_value
+        m_ent_obj.enable.return_value = (False, enable_fail_reason)
+        m_ent_obj.application_status.return_value = (
+            ApplicationStatus.DISABLED,
+            None,
+        )
+        type(m_ent_obj).title = mock.PropertyMock(return_value="Test")
+
+        m_prompt_for_confirmation.return_vale = True
+
+        entitlement = concrete_entitlement_factory(
+            entitled=True,
+            application_status=(ApplicationStatus.DISABLED, ""),
+            required_services=(m_ent_cls,),
+        )
+
+        with mock.patch.object(entitlement, "can_enable") as m_can_enable:
+            m_can_enable.return_value = (False, fail_reason)
+            ret, fail = entitlement.enable()
+
+        assert not ret
+        expected_msg = "Cannot enable required service: Test"
+        if enable_fail_reason.message:
+            expected_msg += "\n" + enable_fail_reason.message.msg
+        assert expected_msg == fail.message.msg
+        assert 1 == m_can_enable.call_count
+
     @pytest.mark.parametrize(
         "orig_access,delta",
         (
@@ -545,15 +692,15 @@ class TestUaEntitlement:
                 {"entitlement": {"entitled": False}},
                 {
                     "entitlement": {
-                        "entitled": False,  # overridden True by series trusty
-                        "series": {"trusty": {"entitled": True}},
+                        "entitled": False,  # overridden by series 'example'
+                        "series": {"example": {"entitled": True}},
                     }
                 },
             ),
         ),
     )
     @mock.patch(
-        "uaclient.util.get_platform_info", return_value={"series": "trusty"}
+        "uaclient.system.get_platform_info", return_value={"series": "example"}
     )
     def test_process_contract_deltas_does_nothing_when_delta_remains_entitled(
         self, m_platform_info, concrete_entitlement_factory, orig_access, delta
@@ -561,12 +708,12 @@ class TestUaEntitlement:
         """If deltas do not represent transition to unentitled, do nothing."""
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
         entitlement.process_contract_deltas(orig_access, delta)
         assert (
-            status.ApplicationStatus.DISABLED,
+            ApplicationStatus.DISABLED,
             mock.ANY,
         ) == entitlement.application_status()
 
@@ -591,7 +738,7 @@ class TestUaEntitlement:
         """Only clear cache when deltas transition inactive to unentitled."""
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
         entitlement.process_contract_deltas(orig_access, delta)
         # If an entitlement is disabled, we don't need to tell the user
@@ -623,11 +770,11 @@ class TestUaEntitlement:
         """Disable when deltas transition from active to unentitled."""
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            application_status=(status.ApplicationStatus.ENABLED, ""),
+            application_status=(ApplicationStatus.ENABLED, ""),
         )
         entitlement.process_contract_deltas(orig_access, delta)
         assert (
-            status.ApplicationStatus.DISABLED,
+            ApplicationStatus.DISABLED,
             mock.ANY,
         ) == entitlement.application_status()
 
@@ -657,8 +804,8 @@ class TestUaEntitlement:
         """Disable when deltas transition from active to unentitled."""
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
-            application_status=(status.ApplicationStatus.DISABLED, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(ApplicationStatus.DISABLED, ""),
         )
         entitlement.is_beta = True
         assert not entitlement.allow_beta
@@ -675,35 +822,85 @@ class TestUaEntitlement:
         self, m_prompt, concrete_entitlement_factory
     ):
         m_prompt.return_value = True
+
+        m_ent_cls = mock.MagicMock()
+        m_ent_obj = m_ent_cls.return_value
+        m_ent_obj.application_status.return_value = [
+            ApplicationStatus.ENABLED,
+            "",
+        ]
+        m_ent_obj.disable.return_value = (True, None)
+        type(m_ent_obj).title = mock.PropertyMock(return_value="test")
         base_ent = concrete_entitlement_factory(
             entitled=True,
             disable=True,
-            application_status=(status.ApplicationStatus.ENABLED, ""),
+            application_status=(ApplicationStatus.ENABLED, ""),
+            dependent_services=(m_ent_cls,),
         )
-        base_ent._dependent_services = ("test",)
 
-        m_entitlement_cls = mock.MagicMock()
-        m_entitlement_obj = m_entitlement_cls.return_value
-        m_entitlement_obj.application_status.return_value = [
-            status.ApplicationStatus.ENABLED,
-            "",
-        ]
-        m_entitlement_obj.disable.return_value = True
-        type(m_entitlement_obj).title = mock.PropertyMock(return_value="test")
-
-        with mock.patch(
-            "uaclient.entitlements.entitlement_factory",
-            return_value=m_entitlement_cls,
-        ):
-            ret = base_ent.disable()
+        ret, fail = base_ent.disable()
 
         expected_prompt_call = 1
         expected_ret = True
         expected_disable_call = 1
 
         assert ret == expected_ret
+        assert fail is None
         assert m_prompt.call_count == expected_prompt_call
-        assert m_entitlement_obj.disable.call_count == expected_disable_call
+        assert m_ent_obj.disable.call_count == expected_disable_call
+
+    @pytest.mark.parametrize("disable_fail_message", (("error"), (None)))
+    @mock.patch("uaclient.util.handle_message_operations")
+    @mock.patch("uaclient.util.prompt_for_confirmation")
+    def test_disable_false_when_fails_to_disable_dependent_service(
+        self,
+        m_handle_msg,
+        m_prompt_for_confirmation,
+        disable_fail_message,
+        concrete_entitlement_factory,
+    ):
+        m_handle_msg.return_value = True
+
+        fail_reason = CanDisableFailure(
+            CanDisableFailureReason.ACTIVE_DEPENDENT_SERVICES
+        )
+        if disable_fail_message:
+            msg = messages.NamedMessage("test-code", disable_fail_message)
+        else:
+            msg = None
+
+        disable_fail_reason = CanDisableFailure(
+            CanDisableFailureReason.ALREADY_DISABLED, message=msg
+        )
+
+        m_ent_cls = mock.Mock()
+        type(m_ent_cls).name = mock.PropertyMock(return_value="Test")
+        m_ent_obj = m_ent_cls.return_value
+        m_ent_obj.disable.return_value = (False, disable_fail_reason)
+        m_ent_obj.application_status.return_value = (
+            ApplicationStatus.ENABLED,
+            None,
+        )
+        type(m_ent_obj).title = mock.PropertyMock(return_value="Test")
+
+        m_prompt_for_confirmation.return_vale = True
+
+        entitlement = concrete_entitlement_factory(
+            entitled=True,
+            application_status=(ApplicationStatus.DISABLED, ""),
+            dependent_services=(m_ent_cls,),
+        )
+
+        with mock.patch.object(entitlement, "can_disable") as m_can_disable:
+            m_can_disable.return_value = (False, fail_reason)
+            ret, fail = entitlement.disable()
+
+        assert not ret
+        expected_msg = "Cannot disable dependent service: Test"
+        if disable_fail_reason.message:
+            expected_msg += "\n" + disable_fail_reason.message.msg
+        assert expected_msg == fail.message.msg
+        assert 1 == m_can_disable.call_count
 
     @pytest.mark.parametrize(
         "p_name,expected",
@@ -734,7 +931,7 @@ class TestUaEntitlement:
             }
         }
         with mock.patch(
-            "uaclient.config.UAConfig.entitlements", m_entitlements
+            "uaclient.files.MachineTokenFile.entitlements", m_entitlements
         ):
             assert "something_else" == entitlement.presentation_name
 
@@ -747,13 +944,13 @@ class TestUaEntitlementUserFacingStatus:
         entitlement = concrete_entitlement_factory(
             entitled=True,
             applicability_status=(
-                status.ApplicabilityStatus.INAPPLICABLE,
+                ApplicabilityStatus.INAPPLICABLE,
                 msg,
             ),
         )
 
         user_facing_status, details = entitlement.user_facing_status()
-        assert status.UserFacingStatus.INAPPLICABLE == user_facing_status
+        assert UserFacingStatus.INAPPLICABLE == user_facing_status
         assert msg == details
 
     def test_unavailable_when_applicable_but_not_entitled(
@@ -762,13 +959,13 @@ class TestUaEntitlementUserFacingStatus:
 
         entitlement = concrete_entitlement_factory(
             entitled=False,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
         )
 
         user_facing_status, details = entitlement.user_facing_status()
-        assert status.UserFacingStatus.UNAVAILABLE == user_facing_status
+        assert UserFacingStatus.UNAVAILABLE == user_facing_status
         expected_details = "{} is not entitled".format(entitlement.title)
-        assert expected_details == details
+        assert expected_details == details.msg
 
     def test_unavailable_when_applicable_but_no_entitlement_cfg(
         self, concrete_entitlement_factory
@@ -776,22 +973,22 @@ class TestUaEntitlementUserFacingStatus:
 
         entitlement = concrete_entitlement_factory(
             entitled=False,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
         )
         entitlement.cfg._entitlements = {}
 
         user_facing_status, details = entitlement.user_facing_status()
-        assert status.UserFacingStatus.UNAVAILABLE == user_facing_status
+        assert UserFacingStatus.UNAVAILABLE == user_facing_status
         expected_details = "{} is not entitled".format(entitlement.title)
-        assert expected_details == details
+        assert expected_details == details.msg
 
     @pytest.mark.parametrize(
         "application_status,expected_uf_status",
         (
-            (status.ApplicationStatus.ENABLED, status.UserFacingStatus.ACTIVE),
+            (ApplicationStatus.ENABLED, UserFacingStatus.ACTIVE),
             (
-                status.ApplicationStatus.DISABLED,
-                status.UserFacingStatus.INACTIVE,
+                ApplicationStatus.DISABLED,
+                UserFacingStatus.INACTIVE,
             ),
         ),
     )
@@ -804,7 +1001,7 @@ class TestUaEntitlementUserFacingStatus:
         msg = "application status details"
         entitlement = concrete_entitlement_factory(
             entitled=True,
-            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
+            applicability_status=(ApplicabilityStatus.APPLICABLE, ""),
             application_status=(application_status, msg),
         )
 
