@@ -8,7 +8,7 @@ from types import MappingProxyType
 import mock
 import pytest
 
-from uaclient import apt, config, status
+from uaclient import apt, messages, status
 from uaclient.entitlements.cc import CC_README, CommonCriteriaEntitlement
 from uaclient.entitlements.tests.conftest import machine_token
 
@@ -54,17 +54,24 @@ class TestCommonCriteriaEntitlementUserFacingStatus:
             ),
             (
                 "s390x",
-                "trusty",
-                "14.04 LTS (Trusty Tahr)",
-                "CC EAL2 is not available for Ubuntu 14.04 LTS"
-                " (Trusty Tahr).",
+                "bionic",
+                "18.04 LTS (Bionic Beaver)",
+                "CC EAL2 is not available for Ubuntu 18.04 LTS"
+                " (Bionic Beaver).",
             ),
         ),
     )
     @mock.patch(M_REPOPATH + "os.getuid", return_value=0)
-    @mock.patch("uaclient.util.get_platform_info")
+    @mock.patch("uaclient.system.get_platform_info")
     def test_inapplicable_on_invalid_affordances(
-        self, m_platform_info, m_getuid, arch, series, version, details, tmpdir
+        self,
+        m_platform_info,
+        m_getuid,
+        arch,
+        series,
+        version,
+        details,
+        FakeConfig,
     ):
         """Test invalid affordances result in inapplicable status."""
         unsupported_info = copy.deepcopy(dict(PLATFORM_INFO_SUPPORTED))
@@ -72,29 +79,31 @@ class TestCommonCriteriaEntitlementUserFacingStatus:
         unsupported_info["series"] = series
         unsupported_info["version"] = version
         m_platform_info.return_value = unsupported_info
-        cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
-        cfg.write_cache("machine-token", CC_MACHINE_TOKEN)
+        cfg = FakeConfig().for_attached_machine(
+            machine_token=CC_MACHINE_TOKEN,
+        )
         entitlement = CommonCriteriaEntitlement(cfg)
         uf_status, uf_status_details = entitlement.user_facing_status()
         assert status.UserFacingStatus.INAPPLICABLE == uf_status
-        assert details == uf_status_details
+        assert details == uf_status_details.msg
 
 
 class TestCommonCriteriaEntitlementCanEnable:
-    @mock.patch("uaclient.util.subp", return_value=("", ""))
-    @mock.patch("uaclient.util.get_platform_info")
+    @mock.patch("uaclient.system.subp", return_value=("", ""))
+    @mock.patch("uaclient.system.get_platform_info")
     def test_can_enable_true_on_entitlement_inactive(
-        self, m_platform_info, _m_subp, capsys, tmpdir
+        self, m_platform_info, _m_subp, capsys, FakeConfig
     ):
         """When entitlement is INACTIVE, can_enable returns True."""
         m_platform_info.return_value = PLATFORM_INFO_SUPPORTED
-        cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
-        cfg.write_cache("machine-token", CC_MACHINE_TOKEN)
+        cfg = FakeConfig().for_attached_machine(
+            machine_token=CC_MACHINE_TOKEN,
+        )
         entitlement = CommonCriteriaEntitlement(cfg, allow_beta=True)
         uf_status, uf_status_details = entitlement.user_facing_status()
         assert status.UserFacingStatus.INACTIVE == uf_status
         details = "{} is not configured".format(entitlement.title)
-        assert details == uf_status_details
+        assert details == uf_status_details.msg
         assert (True, None) == entitlement.can_enable()
         assert ("", "") == capsys.readouterr()
 
@@ -107,22 +116,29 @@ class TestCommonCriteriaEntitlementEnable:
         itertools.product([False, True], repeat=2),
     )
     @mock.patch("uaclient.apt.setup_apt_proxy")
-    @mock.patch("uaclient.util.should_reboot")
-    @mock.patch("uaclient.util.subp")
-    @mock.patch("uaclient.util.get_platform_info")
+    @mock.patch("uaclient.system.should_reboot")
+    @mock.patch("uaclient.system.subp")
+    @mock.patch("uaclient.apt.get_apt_cache_policy")
+    @mock.patch("uaclient.system.get_platform_info")
+    @mock.patch("uaclient.contract.apply_contract_overrides")
     def test_enable_configures_apt_sources_and_auth_files(
         self,
+        _m_contract_overrides,
         m_platform_info,
+        m_apt_cache_policy,
         m_subp,
         m_should_reboot,
         m_setup_apt_proxy,
         capsys,
-        tmpdir,
+        event,
         apt_transport_https,
         ca_certificates,
+        tmpdir,
+        FakeConfig,
     ):
         """When entitled, configure apt repo auth token, pinning and url."""
         m_subp.return_value = ("fakeout", "")
+        m_apt_cache_policy.return_value = "fakeout"
         m_should_reboot.return_value = False
         original_exists = os.path.exists
 
@@ -143,8 +159,9 @@ class TestCommonCriteriaEntitlementEnable:
             return original_exists(path)
 
         m_platform_info.side_effect = fake_platform
-        cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
-        cfg.write_cache("machine-token", CC_MACHINE_TOKEN)
+        cfg = FakeConfig().for_attached_machine(
+            machine_token=CC_MACHINE_TOKEN,
+        )
         entitlement = CommonCriteriaEntitlement(cfg, allow_beta=True)
 
         with mock.patch("uaclient.apt.add_auth_apt_repo") as m_add_apt:
@@ -164,12 +181,9 @@ class TestCommonCriteriaEntitlementEnable:
             )
         ]
 
-        subp_apt_cmds = [
+        apt_cache_policy_cmds = [
             mock.call(
-                ["apt-cache", "policy"],
-                capture=True,
-                retry_sleeps=apt.APT_RETRIES,
-                env={},
+                error_msg=messages.APT_POLICY_FAILED.msg,
             )
         ]
 
@@ -179,6 +193,7 @@ class TestCommonCriteriaEntitlementEnable:
         if ca_certificates:
             prerequisite_pkgs.append("ca-certificates")
 
+        subp_apt_cmds = []
         if prerequisite_pkgs:
             expected_stdout = "Installing prerequisites: {}\n".format(
                 ", ".join(prerequisite_pkgs)
@@ -224,6 +239,8 @@ class TestCommonCriteriaEntitlementEnable:
         assert [] == m_add_pin.call_args_list
         assert 1 == m_setup_apt_proxy.call_count
         assert 1 == m_should_reboot.call_count
+        assert 1 == m_apt_cache_policy.call_count
+        assert apt_cache_policy_cmds == m_apt_cache_policy.call_args_list
         assert subp_apt_cmds == m_subp.call_args_list
         expected_stdout += "\n".join(
             [
