@@ -1,20 +1,23 @@
 """Tests related to uaclient.entitlement.base module."""
 
 import mock
-
 import pytest
 
-from uaclient import apt
-from uaclient import status
-from uaclient.entitlements.cis import CISEntitlement
-
+from uaclient import apt, messages
+from uaclient.entitlements.cis import CIS_DOCS_URL, CISEntitlement
+from uaclient.entitlements.entitlement_status import ApplicationStatus
 
 M_REPOPATH = "uaclient.entitlements.repo."
 
 
 @pytest.fixture
 def entitlement(entitlement_factory):
-    return entitlement_factory(CISEntitlement, additional_packages=["pkg1"])
+    return entitlement_factory(
+        CISEntitlement,
+        allow_beta=True,
+        called_name="cis",
+        additional_packages=["pkg1"],
+    )
 
 
 class TestCISEntitlementCanEnable:
@@ -23,22 +26,31 @@ class TestCISEntitlementCanEnable:
     ):
         """When entitlement is INACTIVE, can_enable returns True."""
         # Unset static affordance container check
-        entitlement.static_affordances = ()
         with mock.patch.object(
             entitlement,
             "application_status",
-            return_value=(status.ApplicationStatus.DISABLED, ""),
+            return_value=(ApplicationStatus.DISABLED, ""),
         ):
             assert entitlement.can_enable()
         assert ("", "") == capsys.readouterr()
 
 
 class TestCISEntitlementEnable:
-    @mock.patch("uaclient.util.should_reboot", return_value=False)
-    @mock.patch("uaclient.util.subp")
-    @mock.patch("uaclient.util.get_platform_info")
+    @mock.patch("uaclient.apt.get_apt_cache_policy")
+    @mock.patch("uaclient.apt.setup_apt_proxy")
+    @mock.patch("uaclient.system.should_reboot")
+    @mock.patch("uaclient.system.subp")
+    @mock.patch("uaclient.system.get_platform_info")
     def test_enable_configures_apt_sources_and_auth_files(
-        self, m_platform_info, m_subp, m_should_reboot, capsys, entitlement
+        self,
+        m_platform_info,
+        m_subp,
+        m_should_reboot,
+        m_setup_apt_proxy,
+        m_apt_policy,
+        capsys,
+        event,
+        entitlement,
     ):
         """When entitled, configure apt repo auth token, pinning and url."""
 
@@ -50,8 +62,8 @@ class TestCISEntitlementEnable:
 
         m_platform_info.side_effect = fake_platform
         m_subp.return_value = ("fakeout", "")
-        # Unset static affordance container check
-        entitlement.static_affordances = ()
+        m_apt_policy.return_value = "fakeout"
+        m_should_reboot.return_value = False
 
         with mock.patch(
             M_REPOPATH + "os.path.exists", mock.Mock(return_value=True)
@@ -62,40 +74,55 @@ class TestCISEntitlementEnable:
 
         add_apt_calls = [
             mock.call(
-                "/etc/apt/sources.list.d/ubuntu-cis-audit-xenial.list",
-                "http://CIS-AUDIT",
+                "/etc/apt/sources.list.d/ubuntu-cis.list",
+                "http://CIS",
                 "{}-token".format(entitlement.name),
                 ["xenial"],
                 entitlement.repo_key_file,
             )
         ]
 
-        subp_apt_cmds = [
+        m_apt_policy_cmds = [
             mock.call(
-                ["apt-cache", "policy"],
-                capture=True,
-                retry_sleeps=apt.APT_RETRIES,
+                error_msg=messages.APT_POLICY_FAILED.msg,
             ),
+        ]
+
+        subp_apt_cmds = [
             mock.call(
                 ["apt-get", "update"],
                 capture=True,
                 retry_sleeps=apt.APT_RETRIES,
+                env={},
             ),
             mock.call(
-                ["apt-get", "install", "--assume-yes"] + entitlement.packages,
+                [
+                    "apt-get",
+                    "install",
+                    "--assume-yes",
+                    "--allow-downgrades",
+                    '-o Dpkg::Options::="--force-confdef"',
+                    '-o Dpkg::Options::="--force-confold"',
+                ]
+                + entitlement.packages,
                 capture=True,
                 retry_sleeps=apt.APT_RETRIES,
+                env={"DEBIAN_FRONTEND": "noninteractive"},
             ),
         ]
 
         assert add_apt_calls == m_add_apt.call_args_list
-        # No apt pinning for cis-audit
+        # No apt pinning for cis
         assert [] == m_add_pin.call_args_list
+        assert 1 == m_setup_apt_proxy.call_count
         assert subp_apt_cmds == m_subp.call_args_list
+        assert 1 == m_apt_policy.call_count
+        assert m_apt_policy_cmds == m_apt_policy.call_args_list
         assert 1 == m_should_reboot.call_count
         expected_stdout = (
             "Updating package lists\n"
             "Installing CIS Audit packages\n"
             "CIS Audit enabled\n"
+            "Visit {} to learn how to use CIS\n".format(CIS_DOCS_URL)
         )
         assert (expected_stdout, "") == capsys.readouterr()
