@@ -9,11 +9,12 @@ from textwrap import dedent
 import mock
 import pytest
 
-from uaclient import apt, exceptions, status, util
+from uaclient import exceptions, messages, system
 from uaclient.apt import (
     APT_AUTH_COMMENT,
-    APT_CONFIG_PROXY_HTTP,
-    APT_CONFIG_PROXY_HTTPS,
+    APT_CONFIG_GLOBAL_PROXY_HTTP,
+    APT_CONFIG_GLOBAL_PROXY_HTTPS,
+    APT_HELPER_TIMEOUT,
     APT_KEYS_DIR,
     APT_PROXY_CONF_FILE,
     APT_RETRIES,
@@ -23,12 +24,17 @@ from uaclient.apt import (
     add_ppa_pinning,
     assert_valid_apt_credentials,
     clean_apt_files,
+    compare_versions,
     find_apt_list_files,
+    get_apt_cache_policy,
+    get_apt_cache_time,
     get_installed_packages,
+    is_installed,
     remove_apt_list_files,
     remove_auth_apt_repo,
     remove_repo_from_apt_auth_file,
-    run_apt_command,
+    restore_commented_apt_list_file,
+    run_apt_update_command,
     setup_apt_proxy,
 )
 from uaclient.entitlements.base import UAEntitlement
@@ -41,9 +47,15 @@ POST_INSTALL_APT_CACHE_NO_UPDATES = """
      origin esm.ubuntu.com
 """
 
+APT_LIST_RETURN_STRING = """\
+"Listing... Done
+a/release, now 1.2+3 arch123 [i,a]
+b/release-updates, now 1.2+3 arch123 [i,a]
+"""
+
 
 class TestAddPPAPinning:
-    @mock.patch("uaclient.util.get_platform_info")
+    @mock.patch("uaclient.system.get_platform_info")
     def test_write_apt_pin_file_to_apt_preferences(self, m_platform, tmpdir):
         """Write proper apt pin file to specified apt_preference_file."""
         m_platform.return_value = {"series": "xenial"}
@@ -57,14 +69,14 @@ class TestAddPPAPinning:
         expected_pref = dedent(
             """\
             Package: *
-            Pin: release o=MYORIG, n=xenial
+            Pin: release o=MYORIG
             Pin-Priority: 1003\n"""
         )
-        assert expected_pref == util.load_file(pref_file)
+        assert expected_pref == system.load_file(pref_file)
 
 
 class TestFindAptListFilesFromRepoSeries:
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     def test_find_all_apt_list_files_from_apt_config_key(self, m_subp, tmpdir):
         """Find all matching apt list files from apt-config dir."""
         m_subp.return_value = ("key='{}'".format(tmpdir.strpath), "")
@@ -81,13 +93,13 @@ class TestFindAptListFilesFromRepoSeries:
             ]
         )
         for path in paths:
-            util.write_file(path, "")
+            system.write_file(path, "")
 
         assert paths[1:] == find_apt_list_files(repo_url, "xenial")
 
 
 class TestRemoveAptListFiles:
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     def test_remove_all_apt_list_files_from_apt_config_key(
         self, m_subp, tmpdir
     ):
@@ -105,14 +117,14 @@ class TestRemoveAptListFiles:
             ).strpath,
         ]
         for path in paths:
-            util.write_file(path, "")
+            system.write_file(path, "")
 
         assert None is remove_apt_list_files(repo_url, "xenial")
         assert [nomatch_file] == glob.glob("{}/*".format(tmpdir.strpath))
 
 
 class TestValidAptCredentials:
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("os.path.exists", return_value=False)
     def test_passes_when_missing_apt_helper(self, m_exists, m_subp):
         """When apt-helper tool is absent perform no validation."""
@@ -124,7 +136,7 @@ class TestValidAptCredentials:
         assert 0 == m_subp.call_count
 
     @mock.patch("uaclient.apt.tempfile.TemporaryDirectory")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.os.path.exists", return_value=True)
     def test_passes_on_valid_creds(
         self, m_exists, m_subp, m_temporary_directory
@@ -167,7 +179,7 @@ class TestValidAptCredentials:
             ),
             (
                 100,
-                "E: Failed to fetch ... HttpError401 on trusty",
+                "E: Failed to fetch ... HttpError401 on xenial",
                 "Invalid APT credentials provided for http://fakerepo",
             ),
             (
@@ -183,7 +195,7 @@ class TestValidAptCredentials:
         ),
     )
     @mock.patch("uaclient.apt.tempfile.TemporaryDirectory")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.os.path.exists", return_value=True)
     def test_errors_on_process_execution_errors(
         self,
@@ -199,7 +211,7 @@ class TestValidAptCredentials:
             "/does/not/exist"
         )
         # Failure apt-helper response
-        m_subp.side_effect = util.ProcessExecutionError(
+        m_subp.side_effect = exceptions.ProcessExecutionError(
             cmd="apt-helper ",
             exit_code=exit_code,
             stdout="Err:1...",
@@ -230,7 +242,7 @@ class TestValidAptCredentials:
         assert [apt_helper_call] == m_subp.call_args_list
 
     @mock.patch("uaclient.apt.tempfile.TemporaryDirectory")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.os.path.exists", return_value=True)
     def test_errors_on_apt_helper_process_timeout(
         self, m_exists, m_subp, m_temporary_directory
@@ -250,7 +262,7 @@ class TestValidAptCredentials:
         error_msg = (
             "Cannot validate credentials for APT repo. Timeout"
             " after {} seconds trying to reach fakerepo.".format(
-                apt.APT_HELPER_TIMEOUT
+                APT_HELPER_TIMEOUT
             )
         )
         assert error_msg == excinfo.value.msg
@@ -267,7 +279,7 @@ class TestValidAptCredentials:
                 "http://user:pwd@fakerepo/ubuntu/pool/",
                 expected_path,
             ],
-            timeout=apt.APT_HELPER_TIMEOUT,
+            timeout=APT_HELPER_TIMEOUT,
             retry_sleeps=APT_RETRIES,
         )
         assert [apt_helper_call] == m_subp.call_args_list
@@ -275,11 +287,11 @@ class TestValidAptCredentials:
 
 class TestAddAuthAptRepo:
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     @mock.patch("uaclient.apt.assert_valid_apt_credentials")
     @mock.patch(
-        "uaclient.util.get_platform_info", return_value={"series": "xenial"}
+        "uaclient.system.get_platform_info", return_value={"series": "xenial"}
     )
     def test_add_auth_apt_repo_writes_sources_file(
         self,
@@ -308,18 +320,18 @@ class TestAddAuthAptRepo:
             "deb http://fakerepo/ubuntu xenial main\n"
             "# deb-src http://fakerepo/ubuntu xenial main\n"
         )
-        assert expected_content == util.load_file(repo_file)
+        assert expected_content == system.load_file(repo_file)
         src_keyfile = os.path.join(KEYRINGS_DIR, "keyring")
         dest_keyfile = os.path.join(APT_KEYS_DIR, "keyring")
         gpg_export_calls = [mock.call(src_keyfile, dest_keyfile)]
         assert gpg_export_calls == m_gpg_export.call_args_list
 
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     @mock.patch("uaclient.apt.assert_valid_apt_credentials")
     @mock.patch(
-        "uaclient.util.get_platform_info", return_value={"series": "xenial"}
+        "uaclient.system.get_platform_info", return_value={"series": "xenial"}
     )
     def test_add_auth_apt_repo_ignores_suites_not_matching_series(
         self,
@@ -360,14 +372,14 @@ class TestAddAuthAptRepo:
             # deb-src http://fakerepo/ubuntu xenial-updates main
         """
         )
-        assert expected_content == util.load_file(repo_file)
+        assert expected_content == system.load_file(repo_file)
 
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     @mock.patch("uaclient.apt.assert_valid_apt_credentials")
     @mock.patch(
-        "uaclient.util.get_platform_info", return_value={"series": "xenial"}
+        "uaclient.system.get_platform_info", return_value={"series": "xenial"}
     )
     def test_add_auth_apt_repo_comments_updates_suites_on_non_update_machine(
         self,
@@ -405,14 +417,14 @@ class TestAddAuthAptRepo:
             # deb-src http://fakerepo/ubuntu xenial-updates main
         """
         )
-        assert expected_content == util.load_file(repo_file)
+        assert expected_content == system.load_file(repo_file)
 
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     @mock.patch("uaclient.apt.assert_valid_apt_credentials")
     @mock.patch(
-        "uaclient.util.get_platform_info", return_value={"series": "xenial"}
+        "uaclient.system.get_platform_info", return_value={"series": "xenial"}
     )
     def test_add_auth_apt_repo_writes_username_password_to_auth_file(
         self,
@@ -441,14 +453,14 @@ class TestAddAuthAptRepo:
             "machine fakerepo/ login user password password"
             "{}\n".format(APT_AUTH_COMMENT)
         )
-        assert expected_content == util.load_file(auth_file)
+        assert expected_content == system.load_file(auth_file)
 
     @mock.patch("uaclient.apt.gpg.export_gpg_key")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     @mock.patch("uaclient.apt.assert_valid_apt_credentials")
     @mock.patch(
-        "uaclient.util.get_platform_info", return_value={"series": "xenial"}
+        "uaclient.system.get_platform_info", return_value={"series": "xenial"}
     )
     def test_add_auth_apt_repo_writes_bearer_resource_token_to_auth_file(
         self,
@@ -477,7 +489,7 @@ class TestAddAuthAptRepo:
             "machine fakerepo/ login bearer password"
             " SOMELONGTOKEN{}\n".format(APT_AUTH_COMMENT)
         )
-        assert expected_content == util.load_file(auth_file)
+        assert expected_content == system.load_file(auth_file)
 
 
 class TestAddAptAuthConfEntry:
@@ -487,7 +499,7 @@ class TestAddAptAuthConfEntry:
     ):
         """Replace old credentials for this repo_url on the same line."""
         auth_file = tmpdir.join("auth.conf").strpath
-        util.write_file(
+        system.write_file(
             auth_file,
             dedent(
                 """\
@@ -512,7 +524,7 @@ class TestAddAptAuthConfEntry:
         """
         )
         expected_content = content_template.format(APT_AUTH_COMMENT)
-        assert expected_content == util.load_file(auth_file)
+        assert expected_content == system.load_file(auth_file)
 
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     def test_insert_repo_subroutes_before_existing_repo_basepath(
@@ -520,7 +532,7 @@ class TestAddAptAuthConfEntry:
     ):
         """Insert new repo_url before first matching url base path."""
         auth_file = tmpdir.join("auth.conf").strpath
-        util.write_file(
+        system.write_file(
             auth_file,
             dedent(
                 """\
@@ -548,7 +560,7 @@ class TestAddAptAuthConfEntry:
         """
         )
         expected_content = content_template.format(APT_AUTH_COMMENT)
-        assert expected_content == util.load_file(auth_file)
+        assert expected_content == system.load_file(auth_file)
 
 
 class TestCleanAptFiles:
@@ -577,14 +589,17 @@ class TestCleanAptFiles:
 
         return TestRepo
 
-    @mock.patch("uaclient.apt.os.unlink")
-    def test_no_removals_for_no_repo_entitlements(self, m_os_unlink):
+    @mock.patch("os.path.exists", return_value=True)
+    @mock.patch("uaclient.system.remove_file")
+    def test_removals_for_repo_entitlements(
+        self, m_remove_file, _m_path_exists
+    ):
         m_entitlements = mock.Mock()
         m_entitlements.ENTITLEMENT_CLASSES = [RepoTestEntitlement]
 
         clean_apt_files(_entitlements=m_entitlements)
 
-        assert 0 == m_os_unlink.call_count
+        assert 2 == m_remove_file.call_count
 
     def test_files_for_all_series_removed(self, mock_apt_entitlement, tmpdir):
         m_entitlements = mock.Mock()
@@ -636,11 +651,11 @@ def remove_auth_apt_repo_kwargs(request):
 
 
 class TestRemoveAuthAptRepo:
-    @mock.patch("uaclient.apt.util.subp")
+    @mock.patch("uaclient.apt.system.subp")
     @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
-    @mock.patch("uaclient.apt.util.del_file")
+    @mock.patch("uaclient.apt.system.remove_file")
     def test_repo_file_deleted(
-        self, m_del_file, _mock, __mock, remove_auth_apt_repo_kwargs
+        self, m_remove_file, _mock, __mock, remove_auth_apt_repo_kwargs
     ):
         """Ensure that repo_filename is deleted, regardless of other params."""
         repo_filename, repo_url = mock.sentinel.filename, mock.sentinel.url
@@ -649,10 +664,10 @@ class TestRemoveAuthAptRepo:
             repo_filename, repo_url, **remove_auth_apt_repo_kwargs
         )
 
-        assert mock.call(repo_filename) in m_del_file.call_args_list
+        assert mock.call(repo_filename) in m_remove_file.call_args_list
 
-    @mock.patch("uaclient.apt.util.subp")
-    @mock.patch("uaclient.apt.util.del_file")
+    @mock.patch("uaclient.apt.system.subp")
+    @mock.patch("uaclient.apt.system.remove_file")
     @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
     def test_remove_from_auth_file_called(
         self, m_remove_repo, _mock, __mock, remove_auth_apt_repo_kwargs
@@ -666,11 +681,11 @@ class TestRemoveAuthAptRepo:
 
         assert mock.call(repo_url) in m_remove_repo.call_args_list
 
-    @mock.patch("uaclient.apt.util.subp")
+    @mock.patch("uaclient.apt.system.subp")
     @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
-    @mock.patch("uaclient.apt.util.del_file")
+    @mock.patch("uaclient.apt.system.remove_file")
     def test_keyring_file_deleted_if_given(
-        self, m_del_file, _mock, __mock, remove_auth_apt_repo_kwargs
+        self, m_remove_file, _mock, __mock, remove_auth_apt_repo_kwargs
     ):
         """We should always delete the keyring file if it is given"""
         repo_filename, repo_url = mock.sentinel.filename, mock.sentinel.url
@@ -683,18 +698,18 @@ class TestRemoveAuthAptRepo:
         if keyring_file:
             assert (
                 mock.call(os.path.join(APT_KEYS_DIR, keyring_file))
-                in m_del_file.call_args_list
+                in m_remove_file.call_args_list
             )
         else:
-            assert mock.call(keyring_file) not in m_del_file.call_args_list
+            assert mock.call(keyring_file) not in m_remove_file.call_args_list
 
 
 class TestRemoveRepoFromAptAuthFile:
-    @mock.patch("uaclient.apt.os.unlink")
-    @mock.patch("uaclient.apt.util.write_file")
+    @mock.patch("uaclient.system.remove_file")
+    @mock.patch("uaclient.apt.system.write_file")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     def test_auth_file_doesnt_exist_means_we_dont_remove_or_write_it(
-        self, m_get_apt_auth_file, m_write_file, m_unlink, tmpdir
+        self, m_get_apt_auth_file, m_write_file, m_remove_file, tmpdir
     ):
         """If the auth file doesn't exist, we shouldn't do anything to it"""
         m_get_apt_auth_file.return_value = tmpdir.join("nonexistent").strpath
@@ -702,7 +717,7 @@ class TestRemoveRepoFromAptAuthFile:
         remove_repo_from_apt_auth_file("http://url")
 
         assert 0 == m_write_file.call_count
-        assert 0 == m_unlink.call_count
+        assert 0 == m_remove_file.call_count
 
     @pytest.mark.parametrize("trailing_slash", (True, False))
     @pytest.mark.parametrize(
@@ -718,14 +733,14 @@ class TestRemoveRepoFromAptAuthFile:
             ),
         ),
     )
-    @mock.patch("uaclient.apt.os.unlink")
-    @mock.patch("uaclient.apt.util.write_file")
+    @mock.patch("uaclient.system.remove_file")
+    @mock.patch("uaclient.apt.system.write_file")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     def test_file_removal(
         self,
         m_get_apt_auth_file,
         m_write_file,
-        m_unlink,
+        m_remove_file,
         tmpdir,
         trailing_slash,
         repo_url,
@@ -741,7 +756,7 @@ class TestRemoveRepoFromAptAuthFile:
         )
 
         assert 0 == m_write_file.call_count
-        assert [mock.call(auth_file.strpath)] == m_unlink.call_args_list
+        assert [mock.call(auth_file.strpath)] == m_remove_file.call_args_list
 
     @pytest.mark.parametrize("trailing_slash", (True, False))
     @pytest.mark.parametrize(
@@ -770,12 +785,12 @@ class TestRemoveRepoFromAptAuthFile:
             ),
         ),
     )
-    @mock.patch("uaclient.apt.os.unlink")
+    @mock.patch("uaclient.system.remove_file")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     def test_file_rewrite(
         self,
         m_get_apt_auth_file,
-        m_unlink,
+        m_remove_file,
         tmpdir,
         repo_url,
         before_content,
@@ -791,7 +806,7 @@ class TestRemoveRepoFromAptAuthFile:
             repo_url + ("" if not trailing_slash else "/")
         )
 
-        assert 0 == m_unlink.call_count
+        assert 0 == m_remove_file.call_count
         assert 0o600 == stat.S_IMODE(os.lstat(auth_file.strpath).st_mode)
         assert after_content == auth_file.read("rb")
 
@@ -825,31 +840,32 @@ class TestRestoreCommentAptListFile:
         list_file = tmpdir.join("list_file")
         list_file.write(before)
 
-        apt.restore_commented_apt_list_file(list_file.strpath)
+        restore_commented_apt_list_file(list_file.strpath)
 
         assert expected == list_file.read()
 
 
 class TestGetInstalledPackages:
-    @mock.patch("uaclient.apt.util.subp", return_value=("", ""))
+    @mock.patch("uaclient.apt.system.subp", return_value=("", ""))
     def test_correct_command_called(self, m_subp):
         get_installed_packages()
 
-        expected_call = mock.call(
-            ["dpkg-query", "-W", "--showformat=${Package}\\n"]
-        )
+        expected_call = mock.call(["apt", "list", "--installed"])
         assert [expected_call] == m_subp.call_args_list
 
-    @mock.patch("uaclient.apt.util.subp", return_value=("", ""))
+    @mock.patch(
+        "uaclient.apt.system.subp", return_value=("Listing... Done\n", "")
+    )
     def test_empty_output_means_empty_list(self, m_subp):
         assert [] == get_installed_packages()
 
-    @mock.patch("uaclient.apt.util.subp", return_value=("a\nb\n", ""))
-    def test_lines_are_split(self, m_subp):
-        assert ["a", "b"] == get_installed_packages()
-
-    @mock.patch("uaclient.apt.util.subp", return_value=("a\nb", ""))
-    def test_assert_missing_eof_newline_works(self, m_subp):
+    @pytest.mark.parametrize(
+        "apt_list_return",
+        (APT_LIST_RETURN_STRING, APT_LIST_RETURN_STRING[:-1]),
+    )
+    @mock.patch("uaclient.apt.system.subp")
+    def test_lines_are_split(self, m_subp, apt_list_return):
+        m_subp.return_value = apt_list_return, ""
         assert ["a", "b"] == get_installed_packages()
 
 
@@ -900,25 +916,61 @@ class TestRunAptCommand:
             ),
         ),
     )
-    @mock.patch("uaclient.apt.util.subp")
+    @mock.patch("uaclient.apt.system.subp")
     def test_run_apt_command_with_invalid_repositories(
         self, m_subp, error_list, output_list
     ):
         error_msg = "\n".join(error_list)
 
-        m_subp.side_effect = util.ProcessExecutionError(
+        m_subp.side_effect = exceptions.ProcessExecutionError(
             cmd="apt update", stderr=error_msg
         )
 
         with pytest.raises(exceptions.UserFacingError) as excinfo:
-            run_apt_command(
-                cmd=["apt", "update"],
-                error_msg=status.MESSAGE_APT_UPDATE_FAILED,
-            )
+            run_apt_update_command()
 
         expected_message = "\n".join(output_list) + "."
         assert expected_message == excinfo.value.msg
 
+    @mock.patch("uaclient.apt.system.subp")
+    def test_run_update_command_clean_apt_cache_policy_cache(self, m_subp):
+        m_subp.side_effect = [
+            ("policy1", ""),
+            ("update", ""),
+            ("policy2", ""),
+        ]
+
+        assert "policy1" == get_apt_cache_policy()
+        # Confirming that caching is happening
+        assert "policy1" == get_apt_cache_policy()
+
+        run_apt_update_command()
+
+        # Confirm cache was cleared
+        assert "policy2" == get_apt_cache_policy()
+        get_apt_cache_policy.cache_clear()
+
+    @mock.patch("uaclient.apt.system.subp")
+    def test_failed_run_update_command_clean_apt_cache_policy_cache(
+        self, m_subp
+    ):
+        m_subp.side_effect = [
+            ("policy1", ""),
+            exceptions.UserFacingError("test"),
+            ("policy2", ""),
+        ]
+
+        assert "policy1" == get_apt_cache_policy()
+        # Confirming that caching is happening
+        assert "policy1" == get_apt_cache_policy()
+
+        with pytest.raises(exceptions.UserFacingError):
+            run_apt_update_command()
+
+        # Confirm cache was cleared
+        assert "policy2" == get_apt_cache_policy()
+        get_apt_cache_policy.cache_clear()
+
 
 class TestAptProxyConfig:
     @pytest.mark.parametrize(
@@ -931,13 +983,13 @@ class TestAptProxyConfig:
                 [
                     mock.call(
                         APT_PROXY_CONF_FILE,
-                        status.MESSAGE_APT_PROXY_CONFIG_HEADER
-                        + APT_CONFIG_PROXY_HTTP.format(
+                        messages.APT_PROXY_CONFIG_HEADER
+                        + APT_CONFIG_GLOBAL_PROXY_HTTP.format(
                             proxy_url="mock_http_proxy"
                         ),
                     )
                 ],
-                status.MESSAGE_SETTING_SERVICE_PROXY.format(service="APT"),
+                messages.SETTING_SERVICE_PROXY_SCOPE.format(scope="global"),
             ),
             (
                 {"https_proxy": "mock_https_proxy"},
@@ -945,13 +997,13 @@ class TestAptProxyConfig:
                 [
                     mock.call(
                         APT_PROXY_CONF_FILE,
-                        status.MESSAGE_APT_PROXY_CONFIG_HEADER
-                        + APT_CONFIG_PROXY_HTTPS.format(
+                        messages.APT_PROXY_CONFIG_HEADER
+                        + APT_CONFIG_GLOBAL_PROXY_HTTPS.format(
                             proxy_url="mock_https_proxy"
                         ),
                     )
                 ],
-                status.MESSAGE_SETTING_SERVICE_PROXY.format(service="APT"),
+                messages.SETTING_SERVICE_PROXY_SCOPE.format(scope="global"),
             ),
             (
                 {
@@ -962,21 +1014,21 @@ class TestAptProxyConfig:
                 [
                     mock.call(
                         APT_PROXY_CONF_FILE,
-                        status.MESSAGE_APT_PROXY_CONFIG_HEADER
-                        + APT_CONFIG_PROXY_HTTP.format(
+                        messages.APT_PROXY_CONFIG_HEADER
+                        + APT_CONFIG_GLOBAL_PROXY_HTTP.format(
                             proxy_url="mock_http_proxy"
                         )
-                        + APT_CONFIG_PROXY_HTTPS.format(
+                        + APT_CONFIG_GLOBAL_PROXY_HTTPS.format(
                             proxy_url="mock_https_proxy"
                         ),
                     )
                 ],
-                status.MESSAGE_SETTING_SERVICE_PROXY.format(service="APT"),
+                messages.SETTING_SERVICE_PROXY_SCOPE.format(scope="global"),
             ),
         ],
     )
-    @mock.patch("uaclient.util.write_file")
-    @mock.patch("uaclient.util.remove_file")
+    @mock.patch("uaclient.system.write_file")
+    @mock.patch("uaclient.system.remove_file")
     def test_setup_apt_proxy_config(
         self,
         m_util_remove_file,
@@ -986,6 +1038,7 @@ class TestAptProxyConfig:
         expected_write_calls,
         expected_out,
         capsys,
+        event,
     ):
         setup_apt_proxy(**kwargs)
         assert expected_remove_calls == m_util_remove_file.call_args_list
@@ -993,3 +1046,54 @@ class TestAptProxyConfig:
         out, err = capsys.readouterr()
         assert expected_out == out.strip()
         assert "" == err
+
+
+class TestAptIsInstalled:
+    @pytest.mark.parametrize(
+        "expected,installed_pkgs",
+        (
+            (True, ("foo", "test", "bar")),
+            (False, ("foo", "bar")),
+        ),
+    )
+    @mock.patch("uaclient.apt.get_installed_packages")
+    def test_is_installed_pkgs(
+        self, m_get_installed_pkgs, expected, installed_pkgs
+    ):
+        m_get_installed_pkgs.return_value = installed_pkgs
+        assert expected == is_installed("test")
+
+
+class TestCompareVersion:
+    @pytest.mark.parametrize(
+        "ver1,ver2,relation,expected_result",
+        (
+            ("1.0", "2.0", "le", True),
+            ("1.0", "2.0", "gt", False),
+            ("2.0", "2.0", "lt", False),
+            ("2.0", "2.0", "eq", True),
+            ("2.0", "2.0", "gt", False),
+            ("2.1~18.04.1", "2.1", "le", True),
+            ("2.1", "2.1~18.04.1", "le", False),
+            ("2.10", "2.9", "ge", True),
+            ("2.10", "2.9", "lt", False),
+        ),
+    )
+    def test_compare_versions(
+        self, ver1, ver2, relation, expected_result, _subp
+    ):
+        """compare_versions returns True when the comparison is accurate."""
+        with mock.patch("uaclient.system._subp", side_effect=_subp):
+            assert expected_result is compare_versions(ver1, ver2, relation)
+
+
+class TestAptCacheTime:
+    @pytest.mark.parametrize(
+        "file_exists,expected", ((True, 1.23), (False, None))
+    )
+    @mock.patch("os.stat")
+    @mock.patch("os.path.exists")
+    def test_get_apt_cache_time(self, m_exists, m_stat, file_exists, expected):
+        m_stat.return_value.st_mtime = 1.23
+        m_exists.return_value = file_exists
+        assert expected == get_apt_cache_time()
