@@ -1,3 +1,4 @@
+import copy
 import datetime
 import io
 import json
@@ -10,8 +11,9 @@ import mock
 import pytest
 import yaml
 
-from uaclient import status, util, version
+from uaclient import exceptions, messages, status
 from uaclient.cli import action_status, get_parser, main, status_parser
+from uaclient.event_logger import EventLoggerMode
 
 M_PATH = "uaclient.cli."
 
@@ -19,9 +21,10 @@ M_PATH = "uaclient.cli."
 RESPONSE_AVAILABLE_SERVICES = [
     {"name": "livepatch", "available": True},
     {"name": "fips", "available": False},
-    {"name": "esm-infra", "available": False},
+    {"name": "esm-infra", "available": True},
     {"name": "esm-apps", "available": True},
     {"name": "fips-updates", "available": False},
+    {"name": "realtime-kernel", "available": False},
     {"name": "ros", "available": False},
     {"name": "ros-updates", "available": False},
 ]
@@ -65,89 +68,103 @@ RESPONSE_CONTRACT_INFO = {
     },
 }
 
+SIMULATED_STATUS_ALL = """\
+SERVICE          AVAILABLE  ENTITLED   AUTO_ENABLED  DESCRIPTION
+esm-apps         yes        no         yes           Expanded Security Maintenance for Applications
+esm-infra        yes        yes        yes           Expanded Security Maintenance for Infrastructure
+fips             no         no         no            NIST-certified core packages
+fips-updates     no         no         no            NIST-certified core packages with priority security updates
+livepatch        yes        yes        no            Canonical Livepatch service
+realtime-kernel  no         no         no            Beta-version Ubuntu Kernel with PREEMPT_RT patches
+ros              no         no         no            Security Updates for the Robot Operating System
+ros-updates      no         no         no            All Updates for the Robot Operating System
+"""  # noqa: E501
+
 SIMULATED_STATUS = """\
-SERVICE       AVAILABLE  ENTITLED   AUTO_ENABLED  DESCRIPTION
-esm-infra     no         yes        yes           UA Infra: Extended Security\
- Maintenance (ESM)
-fips          no         no         no            NIST-certified core packages
-fips-updates  no         no         no            NIST-certified core packages\
- with priority security updates
-livepatch     yes        yes        no            Canonical Livepatch service
-"""
+SERVICE          AVAILABLE  ENTITLED   AUTO_ENABLED  DESCRIPTION
+esm-infra        yes        yes        yes           Expanded Security Maintenance for Infrastructure
+livepatch        yes        yes        no            Canonical Livepatch service
+"""  # noqa: E501
+
+UNATTACHED_STATUS_ALL = """\
+SERVICE          AVAILABLE  DESCRIPTION
+esm-apps         yes        Expanded Security Maintenance for Applications
+esm-infra        yes        Expanded Security Maintenance for Infrastructure
+fips             no         NIST-certified core packages
+fips-updates     no         NIST-certified core packages with priority security updates
+livepatch        yes        Canonical Livepatch service
+realtime-kernel  no         Beta-version Ubuntu Kernel with PREEMPT_RT patches
+ros              no         Security Updates for the Robot Operating System
+ros-updates      no         All Updates for the Robot Operating System
+
+This machine is not attached to an Ubuntu Pro subscription.
+See https://ubuntu.com/pro
+"""  # noqa: E501
 
 UNATTACHED_STATUS = """\
-SERVICE       AVAILABLE  DESCRIPTION
-esm-infra     no         UA Infra: Extended Security Maintenance (ESM)
-fips          no         NIST-certified core packages
-fips-updates  no         NIST-certified core packages with priority\
- security updates
-livepatch     yes        Canonical Livepatch service
+SERVICE          AVAILABLE  DESCRIPTION
+esm-infra        yes        Expanded Security Maintenance for Infrastructure
+livepatch        yes        Canonical Livepatch service
 
-This machine is not attached to a UA subscription.
-See https://ubuntu.com/advantage
-"""
+This machine is not attached to an Ubuntu Pro subscription.
+See https://ubuntu.com/pro
+"""  # noqa: E501
 
-ATTACHED_STATUS = """\
-SERVICE       ENTITLED  STATUS    DESCRIPTION
-esm-apps      no        {dash}         UA Apps: Extended Security Maintenance\
- (ESM)
-esm-infra     no        {dash}         UA Infra: Extended Security Maintenance\
- (ESM)
-fips          no        {dash}         NIST-certified core packages
-fips-updates  no        {dash}         NIST-certified core packages with\
- priority security updates
-livepatch     no        {dash}         Canonical Livepatch service
-ros           no        {dash}         Security Updates for the Robot\
- Operating System
-ros-updates   no        {dash}         All Updates for the Robot Operating\
- System
-{notices}
-Enable services with: ua enable <service>
+ATTACHED_STATUS_ALL = """\
+SERVICE          ENTITLED  STATUS    DESCRIPTION
+esm-apps         no        {dash}         Expanded Security Maintenance for Applications
+esm-infra        no        {dash}         Expanded Security Maintenance for Infrastructure
+fips             no        {dash}         NIST-certified core packages
+fips-updates     no        {dash}         NIST-certified core packages with priority security updates
+livepatch        no        {dash}         Canonical Livepatch service
+realtime-kernel  no        {dash}         Beta-version Ubuntu Kernel with PREEMPT_RT patches
+ros              no        {dash}         Security Updates for the Robot Operating System
+ros-updates      no        {dash}         All Updates for the Robot Operating System
+{notices}{features}
+Enable services with: pro enable <service>
 
                 Account: test_account
            Subscription: test_contract
-            Valid until: 2040-05-08 19:02:26+00:00
+            Valid until: formatteddate
 Technical support level: n/a
-"""
+"""  # noqa: E501
 
 # Omit beta services from status
-ATTACHED_STATUS_NOBETA = """\
-SERVICE       ENTITLED  STATUS    DESCRIPTION
-esm-infra     no        {dash}         UA Infra: Extended Security Maintenance\
- (ESM)
-fips          no        {dash}         NIST-certified core packages
-fips-updates  no        {dash}         NIST-certified core packages with\
- priority security updates
-livepatch     no        {dash}         Canonical Livepatch service
-{notices}
-Enable services with: ua enable <service>
+ATTACHED_STATUS = """\
+SERVICE          ENTITLED  STATUS    DESCRIPTION
+esm-infra        no        {dash}         Expanded Security Maintenance for Infrastructure
+livepatch        no        {dash}         Canonical Livepatch service
+{notices}{features}
+Enable services with: pro enable <service>
 
                 Account: test_account
            Subscription: test_contract
-            Valid until: 2040-05-08 19:02:26+00:00
+            Valid until: formatteddate
 Technical support level: n/a
-"""
+"""  # noqa: E501
 
-BETA_SVC_NAMES = ["esm-apps", "ros", "ros-updates"]
+BETA_SVC_NAMES = ["esm-apps", "realtime-kernel", "ros", "ros-updates"]
 
 SERVICES_JSON_ALL = [
     {
-        "description": "UA Apps: Extended Security Maintenance (ESM)",
+        "description": "Expanded Security Maintenance for Applications",
         "description_override": None,
         "entitled": "no",
         "name": "esm-apps",
         "status": "—",
         "status_details": "",
         "available": "yes",
+        "blocked_by": [],
     },
     {
-        "description": "UA Infra: Extended Security Maintenance (ESM)",
+        "description": "Expanded Security Maintenance for Infrastructure",
         "description_override": None,
         "entitled": "no",
         "name": "esm-infra",
         "status": "—",
         "status_details": "",
         "available": "yes",
+        "blocked_by": [],
     },
     {
         "description": "NIST-certified core packages",
@@ -157,6 +174,7 @@ SERVICES_JSON_ALL = [
         "status": "—",
         "status_details": "",
         "available": "no",
+        "blocked_by": [],
     },
     {
         "description": (
@@ -167,7 +185,8 @@ SERVICES_JSON_ALL = [
         "name": "fips-updates",
         "status": "—",
         "status_details": "",
-        "available": "yes",
+        "available": "no",
+        "blocked_by": [],
     },
     {
         "description": "Canonical Livepatch service",
@@ -177,6 +196,17 @@ SERVICES_JSON_ALL = [
         "status": "—",
         "status_details": "",
         "available": "yes",
+        "blocked_by": [],
+    },
+    {
+        "description": "Beta-version Ubuntu Kernel with PREEMPT_RT patches",
+        "description_override": None,
+        "entitled": "no",
+        "name": "realtime-kernel",
+        "status": "—",
+        "status_details": "",
+        "available": "no",
+        "blocked_by": [],
     },
     {
         "description": "Security Updates for the Robot Operating System",
@@ -185,7 +215,8 @@ SERVICES_JSON_ALL = [
         "name": "ros",
         "status": "—",
         "status_details": "",
-        "available": "yes",
+        "available": "no",
+        "blocked_by": [],
     },
     {
         "description": "All Updates for the Robot Operating System",
@@ -194,15 +225,39 @@ SERVICES_JSON_ALL = [
         "name": "ros-updates",
         "status": "—",
         "status_details": "",
+        "available": "no",
+        "blocked_by": [],
+    },
+]
+
+SERVICES_JSON = [
+    {
+        "description": "Expanded Security Maintenance for Infrastructure",
+        "description_override": None,
+        "entitled": "no",
+        "name": "esm-infra",
+        "status": "—",
+        "status_details": "",
         "available": "yes",
+        "blocked_by": [],
+    },
+    {
+        "description": "Canonical Livepatch service",
+        "description_override": None,
+        "entitled": "no",
+        "name": "livepatch",
+        "status": "—",
+        "status_details": "",
+        "available": "yes",
+        "blocked_by": [],
     },
 ]
 
 HELP_OUTPUT = textwrap.dedent(
     """\
-usage: ua status [flags]
+usage: pro status [flags]
 
-Report current status of Ubuntu Advantage services on system.
+Report current status of Ubuntu Pro services on system.
 
 This shows whether this machine is attached to an Ubuntu Advantage
 support contract. When attached, the report includes the specific
@@ -237,7 +292,7 @@ listed in the output.
 
 Flags:
   -h, --help            show this help message and exit
-  --wait                Block waiting on ua to complete
+  --wait                Block waiting on pro to complete
   --format {tabular,json,yaml}
                         output status in the specified format (default:
                         tabular)
@@ -248,14 +303,15 @@ Flags:
 )
 
 
-@mock.patch("uaclient.util.should_reboot", return_value=False)
-@mock.patch("uaclient.config.UAConfig.remove_notice")
+@mock.patch("uaclient.cli.contract.is_contract_changed", return_value=False)
+@mock.patch("uaclient.files.NoticeFile.remove")
+@mock.patch("uaclient.system.should_reboot", return_value=False)
 @mock.patch(
-    M_PATH + "contract.get_available_resources",
+    "uaclient.status.get_available_resources",
     return_value=RESPONSE_AVAILABLE_SERVICES,
 )
 @mock.patch(
-    M_PATH + "contract.get_contract_information",
+    "uaclient.status.get_contract_information",
     return_value=RESPONSE_CONTRACT_INFO,
 )
 @mock.patch(M_PATH + "os.getuid", return_value=0)
@@ -267,11 +323,17 @@ class TestActionStatus:
         _m_get_available_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
         capsys,
+        FakeConfig,
     ):
         with pytest.raises(SystemExit):
             with mock.patch("sys.argv", ["/usr/bin/ua", "status", "--help"]):
-                main()
+                with mock.patch(
+                    "uaclient.config.UAConfig",
+                    return_value=FakeConfig(),
+                ):
+                    main()
         out, _err = capsys.readouterr()
         assert HELP_OUTPUT == out
 
@@ -286,6 +348,17 @@ class TestActionStatus:
             ),
         ),
     )
+    @pytest.mark.parametrize(
+        "features,feature_status",
+        (
+            ({}, ""),
+            (
+                {"one": True, "other": False, "some": "thing"},
+                "\nFEATURES\none: True\nother: False\nsome: thing\n",
+            ),
+        ),
+    )
+    @mock.patch("uaclient.status.format_expires", return_value="formatteddate")
     def test_attached(
         self,
         _m_getuid,
@@ -293,8 +366,12 @@ class TestActionStatus:
         _m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
+        _m_format_expires,
         notices,
         notice_status,
+        features,
+        feature_status,
         use_all,
         capsys,
         FakeConfig,
@@ -302,9 +379,14 @@ class TestActionStatus:
         """Check that root and non-root will emit attached status"""
         cfg = FakeConfig.for_attached_machine()
         cfg.write_cache("notices", notices)
-        assert 0 == action_status(
-            mock.MagicMock(all=use_all, simulate_with_token=None), cfg=cfg
-        )
+        with mock.patch(
+            "uaclient.config.UAConfig.features",
+            new_callable=mock.PropertyMock,
+            return_value=features,
+        ):
+            assert 0 == action_status(
+                mock.MagicMock(all=use_all, simulate_with_token=None), cfg=cfg
+            )
         # capsys already converts colorized non-printable chars to space
         # Strip non-printables from output
         printable_stdout = capsys.readouterr()[0].replace(" " * 17, " " * 8)
@@ -315,15 +397,26 @@ class TestActionStatus:
         # a specific test that the correct one is used in
         # test_unicode_dash_replacement_when_unprintable
         expected_dash = "-"
-        status_tmpl = ATTACHED_STATUS if use_all else ATTACHED_STATUS_NOBETA
+        status_tmpl = ATTACHED_STATUS_ALL if use_all else ATTACHED_STATUS
 
         if sys.stdout.encoding and "UTF-8" in sys.stdout.encoding.upper():
             expected_dash = "\u2014"
         assert (
-            status_tmpl.format(dash=expected_dash, notices=notice_status)
+            status_tmpl.format(
+                dash=expected_dash,
+                notices=notice_status,
+                features=feature_status,
+            )
             == printable_stdout
         )
 
+    @pytest.mark.parametrize(
+        "use_all",
+        (
+            (True),
+            (False),
+        ),
+    )
     def test_unattached(
         self,
         _m_getuid,
@@ -331,17 +424,27 @@ class TestActionStatus:
         _m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
+        use_all,
         capsys,
         FakeConfig,
     ):
         """Check that unattached status is emitted to console"""
         cfg = FakeConfig()
 
+        expected = UNATTACHED_STATUS_ALL if use_all else UNATTACHED_STATUS
         assert 0 == action_status(
-            mock.MagicMock(all=False, simulate_with_token=None), cfg=cfg
+            mock.MagicMock(all=use_all, simulate_with_token=None), cfg=cfg
         )
-        assert UNATTACHED_STATUS == capsys.readouterr()[0]
+        assert expected == capsys.readouterr()[0]
 
+    @pytest.mark.parametrize(
+        "use_all",
+        (
+            (True),
+            (False),
+        ),
+    )
     def test_simulated(
         self,
         _m_getuid,
@@ -349,20 +452,23 @@ class TestActionStatus:
         _m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
+        use_all,
         capsys,
         FakeConfig,
     ):
         """Check that a simulated status is emitted to console"""
         cfg = FakeConfig()
+        expected = SIMULATED_STATUS_ALL if use_all else SIMULATED_STATUS
 
         assert 0 == action_status(
-            mock.MagicMock(all=False, simulate_with_token="some_token"),
+            mock.MagicMock(all=use_all, simulate_with_token="some_token"),
             cfg=cfg,
         )
-        assert SIMULATED_STATUS == capsys.readouterr()[0]
+        assert expected == capsys.readouterr()[0]
 
     @mock.patch("uaclient.version.get_version", return_value="test_version")
-    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.system.subp")
     @mock.patch(M_PATH + "time.sleep")
     def test_wait_blocks_until_lock_released(
         self,
@@ -374,13 +480,14 @@ class TestActionStatus:
         _m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
         capsys,
         FakeConfig,
     ):
         """Check that --wait will will block and poll until lock released."""
         cfg = FakeConfig()
         lock_file = cfg.data_path("lock")
-        cfg.write_cache("lock", "123:ua auto-attach")
+        cfg.write_cache("lock", "123:pro auto-attach")
 
         def fake_sleep(seconds):
             if m_sleep.call_count == 3:
@@ -394,7 +501,17 @@ class TestActionStatus:
         assert [mock.call(1)] * 3 == m_sleep.call_args_list
         assert "...\n" + UNATTACHED_STATUS == capsys.readouterr()[0]
 
-    @pytest.mark.parametrize("format_type", ("json", "yaml"))
+    @pytest.mark.parametrize(
+        "use_all",
+        (
+            (True),
+            (False),
+        ),
+    )
+    @pytest.mark.parametrize(
+        "format_type,event_logger_mode",
+        (("json", EventLoggerMode.JSON), ("yaml", EventLoggerMode.YAML)),
+    )
     @pytest.mark.parametrize(
         "environ",
         (
@@ -407,7 +524,6 @@ class TestActionStatus:
             },
         ),
     )
-    @pytest.mark.parametrize("use_all", (True, False))
     def test_unattached_formats(
         self,
         _m_getuid,
@@ -415,11 +531,14 @@ class TestActionStatus:
         _m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
-        use_all,
+        _m_contract_changed,
         environ,
         format_type,
+        event_logger_mode,
+        use_all,
         capsys,
         FakeConfig,
+        event,
     ):
         """Check that unattached status json output is emitted to console"""
         cfg = FakeConfig()
@@ -428,7 +547,10 @@ class TestActionStatus:
             format=format_type, all=use_all, simulate_with_token=None
         )
         with mock.patch.object(os, "environ", environ):
-            assert 0 == action_status(args, cfg=cfg)
+            with mock.patch.object(
+                event, "_event_logger_mode", event_logger_mode
+            ), mock.patch.object(event, "_command", "status"):
+                assert 0 == action_status(args, cfg=cfg)
 
         expected_environment = []
         if environ:
@@ -438,20 +560,15 @@ class TestActionStatus:
                 {"name": "UA_FEATURES_ALLOW_BETA", "value": "true"},
             ]
 
-        expected_services = [
+        services = SERVICES_JSON_ALL if use_all else SERVICES_JSON
+        services = [
             {
-                "name": "esm-apps",
-                "description": "UA Apps: Extended Security Maintenance (ESM)",
-                "available": "yes",
-            },
-            {
-                "name": "livepatch",
-                "description": "Canonical Livepatch service",
-                "available": "yes",
-            },
+                "name": service["name"],
+                "description": service["description"],
+                "available": service["available"],
+            }
+            for service in services
         ]
-        if not use_all:
-            expected_services.pop(0)
 
         expected = {
             "_doc": (
@@ -459,15 +576,16 @@ class TestActionStatus:
                 "considered Experimental and may change"
             ),
             "_schema_version": "0.1",
-            "version": version.get_version(features=cfg.features),
+            "version": mock.ANY,
             "execution_status": status.UserFacingConfigStatus.INACTIVE.value,
-            "execution_details": status.MESSAGE_NO_ACTIVE_OPERATIONS,
+            "execution_details": messages.NO_ACTIVE_OPERATIONS,
             "attached": False,
             "machine_id": None,
             "effective": None,
             "expires": None,
+            "features": {},
             "notices": [],
-            "services": expected_services,
+            "services": services,
             "environment_vars": expected_environment,
             "contract": {
                 "id": "",
@@ -485,6 +603,9 @@ class TestActionStatus:
             "config_path": None,
             "config": {"data_dir": mock.ANY},
             "simulated": False,
+            "errors": [],
+            "warnings": [],
+            "result": "success",
         }
 
         if format_type == "json":
@@ -492,7 +613,10 @@ class TestActionStatus:
         else:
             assert expected == yaml.safe_load(capsys.readouterr()[0])
 
-    @pytest.mark.parametrize("format_type", ("json", "yaml"))
+    @pytest.mark.parametrize(
+        "format_type,event_logger_mode",
+        (("json", EventLoggerMode.JSON), ("yaml", EventLoggerMode.YAML)),
+    )
     @pytest.mark.parametrize(
         "environ",
         (
@@ -513,11 +637,14 @@ class TestActionStatus:
         _m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
         use_all,
         environ,
         format_type,
+        event_logger_mode,
         capsys,
         FakeConfig,
+        event,
     ):
         """Check that unattached status json output is emitted to console"""
         cfg = FakeConfig.for_attached_machine()
@@ -527,7 +654,13 @@ class TestActionStatus:
         )
 
         with mock.patch.object(os, "environ", environ):
-            assert 0 == action_status(args, cfg=cfg)
+            with mock.patch.object(
+                event, "_event_logger_mode", event_logger_mode
+            ), mock.patch.object(event, "_command", "status"):
+                with mock.patch(
+                    "uaclient.status._get_blocked_by_services", return_value=[]
+                ):
+                    assert 0 == action_status(args, cfg=cfg)
 
         expected_environment = []
         if environ:
@@ -537,26 +670,7 @@ class TestActionStatus:
                 {"name": "UA_FEATURES_ALLOW_BETA", "value": "true"},
             ]
 
-        if use_all:
-            services = SERVICES_JSON_ALL
-        else:
-            services = [
-                svc
-                for svc in SERVICES_JSON_ALL
-                if svc["name"] not in BETA_SVC_NAMES
-            ]
-
-        inapplicable_services = [
-            service["name"]
-            for service in RESPONSE_AVAILABLE_SERVICES
-            if not service["available"]
-        ]
-
-        filtered_services = [
-            service
-            for service in services
-            if service["name"] not in inapplicable_services
-        ]
+        services = SERVICES_JSON_ALL if use_all else SERVICES_JSON
 
         if format_type == "json":
             contract_created_at = "2020-05-08T19:02:26+00:00"
@@ -584,15 +698,16 @@ class TestActionStatus:
                 "considered Experimental and may change"
             ),
             "_schema_version": "0.1",
-            "version": version.get_version(features=cfg.features),
+            "version": mock.ANY,
             "execution_status": status.UserFacingConfigStatus.INACTIVE.value,
-            "execution_details": status.MESSAGE_NO_ACTIVE_OPERATIONS,
+            "execution_details": messages.NO_ACTIVE_OPERATIONS,
             "attached": True,
             "machine_id": "test_machine_id",
             "effective": effective,
             "expires": expires,
+            "features": {},
             "notices": [],
-            "services": filtered_services,
+            "services": services,
             "environment_vars": expected_environment,
             "contract": {
                 "id": "cid",
@@ -605,11 +720,14 @@ class TestActionStatus:
                 "id": "acct-1",
                 "name": "test_account",
                 "created_at": account_created_at,
-                "external_account_ids": [{"IDs": ["id1"], "Origin": "AWS"}],
+                "external_account_ids": [{"IDs": ["id1"], "origin": "AWS"}],
             },
             "config_path": None,
             "config": {"data_dir": mock.ANY},
             "simulated": False,
+            "errors": [],
+            "warnings": [],
+            "result": "success",
         }
 
         if format_type == "json":
@@ -637,7 +755,10 @@ class TestActionStatus:
 
             assert expected == yaml_output
 
-    @pytest.mark.parametrize("format_type", ("json", "yaml"))
+    @pytest.mark.parametrize(
+        "format_type,event_logger_mode",
+        (("json", EventLoggerMode.JSON), ("yaml", EventLoggerMode.YAML)),
+    )
     @pytest.mark.parametrize("use_all", (True, False))
     def test_simulated_formats(
         self,
@@ -646,32 +767,64 @@ class TestActionStatus:
         _m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
         use_all,
         format_type,
+        event_logger_mode,
         capsys,
         FakeConfig,
+        event,
     ):
-        """Check that unattached status json output is emitted to console"""
+        """Check that simulated status json output is emitted to console"""
         cfg = FakeConfig()
 
         args = mock.MagicMock(
             format=format_type, all=use_all, simulate_with_token="some_token"
         )
 
-        assert 0 == action_status(args, cfg=cfg)
+        with mock.patch.object(
+            event, "_event_logger_mode", event_logger_mode
+        ), mock.patch.object(event, "_command", "status"):
+            assert 0 == action_status(args, cfg=cfg)
 
-        expected_services = [
+        beta_services = [
+            {
+                "auto_enabled": "no",
+                "available": "no",
+                "description": "Beta-version Ubuntu Kernel with PREEMPT_RT"
+                " patches",
+                "entitled": "no",
+                "name": "realtime-kernel",
+            },
             {
                 "auto_enabled": "yes",
                 "available": "yes",
-                "description": "UA Apps: Extended Security Maintenance (ESM)",
+                "description": "Expanded Security Maintenance for Applications",  # noqa
                 "entitled": "no",
                 "name": "esm-apps",
             },
             {
-                "auto_enabled": "yes",
+                "auto_enabled": "no",
                 "available": "no",
-                "description": "UA Infra: Extended Security Maintenance (ESM)",
+                "description": "Security Updates for the Robot Operating"
+                " System",
+                "entitled": "no",
+                "name": "ros",
+            },
+            {
+                "auto_enabled": "no",
+                "available": "no",
+                "description": "All Updates for the Robot Operating System",
+                "entitled": "no",
+                "name": "ros-updates",
+            },
+        ]
+
+        services = [
+            {
+                "auto_enabled": "yes",
+                "available": "yes",
+                "description": "Expanded Security Maintenance for Infrastructure",  # noqa
                 "entitled": "yes",
                 "name": "esm-infra",
             },
@@ -697,25 +850,17 @@ class TestActionStatus:
                 "entitled": "yes",
                 "name": "livepatch",
             },
-            {
-                "auto_enabled": "no",
-                "available": "no",
-                "description": "Security Updates for the Robot Operating"
-                " System",
-                "entitled": "no",
-                "name": "ros",
-            },
-            {
-                "auto_enabled": "no",
-                "available": "no",
-                "description": "All Updates for the Robot Operating System",
-                "entitled": "no",
-                "name": "ros-updates",
-            },
         ]
 
+        expected_services = sorted(
+            services + beta_services, key=lambda x: x["name"]
+        )
         if not use_all:
-            expected_services = expected_services[1:-2]
+            expected_services = [
+                service
+                for service in services
+                if service["available"] == "yes"
+            ]
 
         expected = {
             "_doc": "Content provided in json response is currently considered"
@@ -723,6 +868,7 @@ class TestActionStatus:
             "_schema_version": "0.1",
             "attached": False,
             "machine_id": None,
+            "features": {},
             "notices": [],
             "account": {
                 "created_at": "2019-06-14T06:45:50Z",
@@ -739,14 +885,17 @@ class TestActionStatus:
             },
             "environment_vars": [],
             "execution_status": "inactive",
-            "execution_details": "No Ubuntu Advantage operations are running",
+            "execution_details": "No Ubuntu Pro operations are running",
             "expires": "9999-12-31T00:00:00Z",
             "effective": None,
             "services": expected_services,
             "simulated": True,
-            "version": version.get_version(features=cfg.features),
+            "version": mock.ANY,
             "config_path": None,
             "config": {"data_dir": mock.ANY},
+            "errors": [],
+            "warnings": [],
+            "result": "success",
         }
 
         if format_type == "json":
@@ -761,24 +910,27 @@ class TestActionStatus:
         m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
         FakeConfig,
     ):
         """Raise UrlError on connectivity issues"""
-        m_get_avail_resources.side_effect = util.UrlError(
+        m_get_avail_resources.side_effect = exceptions.UrlError(
             socket.gaierror(-2, "Name or service not known")
         )
 
         cfg = FakeConfig()
 
-        with pytest.raises(util.UrlError):
+        with pytest.raises(exceptions.UrlError):
             action_status(
                 mock.MagicMock(all=False, simulate_with_token=None), cfg=cfg
             )
 
+    @pytest.mark.parametrize("use_all", (True, False))
     @pytest.mark.parametrize(
         "encoding,expected_dash",
         (("utf-8", "\u2014"), ("UTF-8", "\u2014"), ("ascii", "-")),
     )
+    @mock.patch("uaclient.status.format_expires", return_value="formatteddate")
     def test_unicode_dash_replacement_when_unprintable(
         self,
         _m_getuid,
@@ -786,8 +938,11 @@ class TestActionStatus:
         _m_get_avail_resources,
         _m_should_reboot,
         _m_remove_notice,
+        _m_contract_changed,
+        _m_format_expires,
         encoding,
         expected_dash,
+        use_all,
         FakeConfig,
     ):
         # This test can't use capsys because it doesn't emulate sys.stdout
@@ -797,7 +952,7 @@ class TestActionStatus:
 
         with mock.patch("sys.stdout", fake_stdout):
             action_status(
-                mock.MagicMock(all=True, simulate_with_token=None),
+                mock.MagicMock(all=use_all, simulate_with_token=None),
                 cfg=FakeConfig.for_attached_machine(),
             )
 
@@ -808,22 +963,192 @@ class TestActionStatus:
         # comparison
         out = out.replace(" " * 17, " " * 8)
 
-        expected_out = ATTACHED_STATUS.format(dash=expected_dash, notices="")
+        if not use_all:
+            expected_out = ATTACHED_STATUS.format(
+                dash=expected_dash, notices="", features=""
+            )
+        else:
+            expected_out = ATTACHED_STATUS_ALL.format(
+                dash=expected_dash, notices="", features=""
+            )
+
         assert expected_out == out
 
+    @pytest.mark.parametrize(
+        "exception_to_throw,exception_type,exception_message",
+        (
+            (
+                exceptions.UrlError("Not found", 404),
+                exceptions.UrlError,
+                "Not found",
+            ),
+            (
+                exceptions.ContractAPIError(
+                    exceptions.UrlError("Unauthorized", 401),
+                    {"message": "unauthorized"},
+                ),
+                exceptions.UserFacingError,
+                "Invalid token. See https://ubuntu.com/pro",
+            ),
+        ),
+    )
+    def test_errors_are_raised_appropriately(
+        self,
+        _m_getuid,
+        m_get_contract_information,
+        _m_get_avail_resources,
+        _m_should_reboot,
+        _m_remove_notice,
+        _m_contract_changed,
+        exception_to_throw,
+        exception_type,
+        exception_message,
+        capsys,
+        FakeConfig,
+    ):
+        """Check that simulated status json/yaml output raises errors."""
+
+        m_get_contract_information.side_effect = exception_to_throw
+
+        cfg = FakeConfig()
+
+        args = mock.MagicMock(
+            format="json", all=False, simulate_with_token="some_token"
+        )
+
+        with pytest.raises(exception_type) as exc:
+            action_status(args, cfg=cfg)
+
+        assert exc.type == exception_type
+        assert exception_message in getattr(exc.value, "msg", exc.value.args)
+
+    @pytest.mark.parametrize(
+        "token_to_use,warning_message,contract_field,date_value",
+        (
+            (
+                "expired_token",
+                'Contract "some_id" expired on December 31, 2019',
+                "effectiveTo",
+                "2019-12-31T00:00:00Z",
+            ),
+            (
+                "token_not_valid_yet",
+                'Contract "some_id" is not effective until December 31, 9999',
+                "effectiveFrom",
+                "9999-12-31T00:00:00Z",
+            ),
+        ),
+    )
+    @pytest.mark.parametrize(
+        "format_type,event_logger_mode",
+        (("json", EventLoggerMode.JSON), ("yaml", EventLoggerMode.YAML)),
+    )
+    def test_errors_for_token_dates(
+        self,
+        _m_getuid,
+        m_get_contract_information,
+        _m_get_avail_resources,
+        _m_should_reboot,
+        _m_remove_notice,
+        _m_contract_changed,
+        format_type,
+        event_logger_mode,
+        token_to_use,
+        warning_message,
+        contract_field,
+        date_value,
+        capsys,
+        FakeConfig,
+        event,
+    ):
+        """Check errors for expired tokens, and not valid yet tokens."""
+
+        def contract_info_side_effect(cfg, token):
+            response = copy.deepcopy(RESPONSE_CONTRACT_INFO)
+            response["contractInfo"][contract_field] = date_value
+            return response
+
+        m_get_contract_information.side_effect = contract_info_side_effect
+
+        cfg = FakeConfig()
+
+        args = mock.MagicMock(
+            format=format_type, all=False, simulate_with_token=token_to_use
+        )
+
+        with mock.patch.object(
+            event, "_event_logger_mode", event_logger_mode
+        ), mock.patch.object(event, "_command", "status"):
+            assert 1 == action_status(args, cfg=cfg)
+
+        if format_type == "json":
+            output = json.loads(capsys.readouterr()[0])
+        else:
+            output = yaml.safe_load(capsys.readouterr()[0])
+
+        assert output["errors"][0]["message"] == warning_message
+
+    @pytest.mark.parametrize(
+        "contract_changed,is_attached",
+        (
+            (False, True),
+            (True, False),
+            (True, True),
+            (False, False),
+        ),
+    )
+    @mock.patch("uaclient.files.NoticeFile.add")
+    def test_is_contract_changed(
+        self,
+        m_add_notice,
+        _m_getuid,
+        _m_get_contract_information,
+        _m_get_available_resources,
+        _m_should_reboot,
+        _m_remove_notice,
+        _m_contract_changed,
+        contract_changed,
+        is_attached,
+        capsys,
+        FakeConfig,
+    ):
+        _m_contract_changed.return_value = contract_changed
+        if is_attached:
+            cfg = FakeConfig().for_attached_machine()
+        else:
+            cfg = FakeConfig()
+
+        action_status(
+            mock.MagicMock(all=False, simulate_with_token=None), cfg=cfg
+        )
+
+        if is_attached:
+            if contract_changed:
+                assert [
+                    mock.call("", messages.NOTICE_REFRESH_CONTRACT_WARNING)
+                ] == m_add_notice.call_args_list
+            else:
+                assert [
+                    mock.call("", messages.NOTICE_REFRESH_CONTRACT_WARNING)
+                ] not in m_add_notice.call_args_list
+        else:
+            assert _m_contract_changed.call_count == 0
+
 
 class TestStatusParser:
     @mock.patch(M_PATH + "contract.get_available_resources")
-    def test_status_parser_updates_parser_config(self, _m_resources):
+    def test_status_parser_updates_parser_config(
+        self, _m_resources, FakeConfig
+    ):
         """Update the parser configuration for 'status'."""
         m_parser = status_parser(mock.Mock())
         assert "status" == m_parser.prog
 
-        full_parser = get_parser()
+        full_parser = get_parser(FakeConfig())
         with mock.patch(
             "sys.argv",
             [
-                "ua",
+                "pro",
                 "status",
                 "--format",
                 "json",
