@@ -9,8 +9,12 @@ from lib.reboot_cmds import (
     process_reboot_operations,
     run_command,
 )
-from uaclient.status import MESSAGE_REBOOT_SCRIPT_FAILED
-from uaclient.util import ProcessExecutionError
+from uaclient.exceptions import ProcessExecutionError
+from uaclient.messages import (
+    FIPS_REBOOT_REQUIRED_MSG,
+    FIPS_SYSTEM_REBOOT_REQUIRED,
+    REBOOT_SCRIPT_FAILED,
+)
 
 M_FIPS_PATH = "uaclient.entitlements.fips.FIPSEntitlement."
 
@@ -23,7 +27,7 @@ class TestMain:
             with mock.patch(
                 "uaclient.config.UAConfig.check_lock_info"
             ) as m_check_lock:
-                m_check_lock.return_value = (123, "ua auto-attach")
+                m_check_lock.return_value = (123, "pro auto-attach")
                 with mock.patch("time.sleep") as m_sleep:
                     main(cfg=cfg)
         assert [
@@ -77,8 +81,10 @@ class TestFixProPkgHolds:
     @mock.patch("sys.exit")
     @mock.patch(M_FIPS_PATH + "install_packages")
     @mock.patch(M_FIPS_PATH + "setup_apt_config")
+    @mock.patch("uaclient.files.NoticeFile.remove")
     def test_calls_setup_apt_config_and_install_packages_when_enabled(
         self,
+        m_remove_notice,
         setup_apt_config,
         install_packages,
         exit,
@@ -97,9 +103,14 @@ class TestFixProPkgHolds:
             assert [
                 mock.call(cleanup_on_failure=False)
             ] == install_packages.call_args_list
+            assert [
+                mock.call("", FIPS_SYSTEM_REBOOT_REQUIRED.msg),
+                mock.call("", FIPS_REBOOT_REQUIRED_MSG),
+            ] == m_remove_notice.call_args_list
         else:
             assert 0 == setup_apt_config.call_count
             assert 0 == install_packages.call_count
+            assert 0 == len(m_remove_notice.call_args_list)
         assert 0 == exit.call_count
 
 
@@ -137,7 +148,7 @@ class TestProcessRebootOperations:
     @pytest.mark.parametrize("caplog_text", [logging.ERROR], indirect=True)
     @mock.patch("uaclient.config.UAConfig.delete_cache_key")
     @mock.patch("uaclient.config.UAConfig.check_lock_info")
-    @mock.patch("uaclient.config.UAConfig.add_notice")
+    @mock.patch("uaclient.files.NoticeFile.add")
     @mock.patch("lib.reboot_cmds.fix_pro_pkg_holds")
     def test_process_reboot_operations_create_notice_when_it_fails(
         self,
@@ -157,7 +168,7 @@ class TestProcessRebootOperations:
             with mock.patch("uaclient.config.UAConfig.write_cache"):
                 process_reboot_operations(cfg=cfg)
 
-        expected_calls = [mock.call("", MESSAGE_REBOOT_SCRIPT_FAILED)]
+        expected_calls = [mock.call("", REBOOT_SCRIPT_FAILED)]
 
         assert expected_calls == m_add_notice.call_args_list
 
